@@ -20,6 +20,7 @@
  * @author Marcello Stanisci
  * @author Christian Grothoff
  * @author Florian Dold
+ * @author Priscilla HUANG
  */
 #include "platform.h"
 #include <taler/taler_dbevents.h>
@@ -33,11 +34,17 @@
 #include "taler-merchant-httpd_get-orders-ID.h"
 #include "taler-merchant-httpd_get-tips-ID.h"
 #include "taler-merchant-httpd_mhd.h"
+#include "taler-merchant-httpd_private-delete-webhooks-ID.h"
+#include "taler-merchant-httpd_private-delete-templates-ID.h"
 #include "taler-merchant-httpd_private-delete-instances-ID.h"
 #include "taler-merchant-httpd_private-delete-products-ID.h"
 #include "taler-merchant-httpd_private-delete-orders-ID.h"
 #include "taler-merchant-httpd_private-delete-reserves-ID.h"
 #include "taler-merchant-httpd_private-delete-transfers-ID.h"
+#include "taler-merchant-httpd_private-get-webhooks.h"
+#include "taler-merchant-httpd_private-get-webhooks-ID.h"
+#include "taler-merchant-httpd_private-get-templates.h"
+#include "taler-merchant-httpd_private-get-templates-ID.h"
 #include "taler-merchant-httpd_private-get-instances.h"
 #include "taler-merchant-httpd_private-get-instances-ID.h"
 #include "taler-merchant-httpd_private-get-instances-ID-kyc.h"
@@ -50,9 +57,13 @@
 #include "taler-merchant-httpd_private-get-tips-ID.h"
 #include "taler-merchant-httpd_private-get-tips.h"
 #include "taler-merchant-httpd_private-get-transfers.h"
+#include "taler-merchant-httpd_private-patch-webhooks-ID.h"
+#include "taler-merchant-httpd_private-patch-templates-ID.h"
 #include "taler-merchant-httpd_private-patch-instances-ID.h"
 #include "taler-merchant-httpd_private-patch-orders-ID-forget.h"
 #include "taler-merchant-httpd_private-patch-products-ID.h"
+#include "taler-merchant-httpd_private-post-webhooks.h"
+#include "taler-merchant-httpd_private-post-templates.h"
 #include "taler-merchant-httpd_private-post-instances.h"
 #include "taler-merchant-httpd_private-post-instances-ID-auth.h"
 #include "taler-merchant-httpd_private-post-orders.h"
@@ -66,6 +77,7 @@
 #include "taler-merchant-httpd_post-orders-ID-claim.h"
 #include "taler-merchant-httpd_post-orders-ID-paid.h"
 #include "taler-merchant-httpd_post-orders-ID-pay.h"
+#include "taler-merchant-httpd_post-using-templates.h"
 #include "taler-merchant-httpd_post-orders-ID-refund.h"
 #include "taler-merchant-httpd_post-tips-ID-pickup.h"
 #include "taler-merchant-httpd_reserves.h"
@@ -447,6 +459,49 @@ handle_server_options (const struct TMH_RequestHandler *rh,
 }
 
 
+static MHD_RESULT
+spa_redirect (const struct TMH_RequestHandler *rh,
+              struct MHD_Connection *connection,
+              struct TMH_HandlerContext *hc)
+{
+  const char *text = "Redirecting to /webui/";
+  struct MHD_Response *response;
+
+  response = MHD_create_response_from_buffer (strlen (text),
+                                              (void *) text,
+                                              MHD_RESPMEM_PERSISTENT);
+  if (NULL == response)
+  {
+    GNUNET_break (0);
+    return MHD_NO;
+  }
+  TALER_MHD_add_global_headers (response);
+  GNUNET_break (MHD_YES ==
+                MHD_add_response_header (response,
+                                         MHD_HTTP_HEADER_CONTENT_TYPE,
+                                         "text/plain"));
+  if (MHD_NO ==
+      MHD_add_response_header (response,
+                               MHD_HTTP_HEADER_LOCATION,
+                               "/webui/"))
+  {
+    GNUNET_break (0);
+    MHD_destroy_response (response);
+    return MHD_NO;
+  }
+
+  {
+    MHD_RESULT ret;
+
+    ret = MHD_queue_response (connection,
+                              MHD_HTTP_FOUND,
+                              response);
+    MHD_destroy_response (response);
+    return ret;
+  }
+}
+
+
 /**
  * Extract the token from authorization header value @a auth.
  *
@@ -920,6 +975,98 @@ url_handler (void *cls,
       .allow_deleted_instance = true,
       .handler = &TMH_private_get_transfers
     },
+    /* POST /templates: */
+    {
+      .url_prefix = "/templates",
+      .method = MHD_HTTP_METHOD_POST,
+      .handler = &TMH_private_post_templates,
+      /* allow template data of up to 8 MB, that should be plenty;
+         note that exceeding #GNUNET_MAX_MALLOC_CHECKED (40 MB)
+         would require further changes to the allocation logic
+         in the code... */
+      .max_upload = 1024 * 1024 * 8
+    },
+    /* GET /templates: */
+    {
+      .url_prefix = "/templates",
+      .method = MHD_HTTP_METHOD_GET,
+      .handler = &TMH_private_get_templates
+    },
+    /* GET /templates/$ID/: */
+    {
+      .url_prefix = "/templates/",
+      .method = MHD_HTTP_METHOD_GET,
+      .have_id_segment = true,
+      .allow_deleted_instance = true,
+      .handler = &TMH_private_get_templates_ID
+    },
+    /* DELETE /templates/$ID/: */
+    {
+      .url_prefix = "/templates/",
+      .method = MHD_HTTP_METHOD_DELETE,
+      .have_id_segment = true,
+      .allow_deleted_instance = true,
+      .handler = &TMH_private_delete_templates_ID
+    },
+    /* PATCH /templates/$ID/: */
+    {
+      .url_prefix = "/templates/",
+      .method = MHD_HTTP_METHOD_PATCH,
+      .have_id_segment = true,
+      .allow_deleted_instance = true,
+      .handler = &TMH_private_patch_templates_ID,
+      /* allow template data of up to 8 MB, that should be plenty;
+         note that exceeding #GNUNET_MAX_MALLOC_CHECKED (40 MB)
+         would require further changes to the allocation logic
+         in the code... */
+      .max_upload = 1024 * 1024 * 8
+    },
+    /* GET /webhooks: */
+    {
+      .url_prefix = "/webhooks",
+      .method = MHD_HTTP_METHOD_GET,
+      .handler = &TMH_private_get_webhooks
+    },
+    /* POST /webhooks: */
+    {
+      .url_prefix = "/webhooks",
+      .method = MHD_HTTP_METHOD_POST,
+      .handler = &TMH_private_post_webhooks,
+      /* allow webhook data of up to 8 MB, that should be plenty;
+         note that exceeding #GNUNET_MAX_MALLOC_CHECKED (40 MB)
+         would require further changes to the allocation logic
+         in the code... */
+      .max_upload = 1024 * 1024 * 8
+    },
+    /* GET /webhooks/$ID/: */
+    {
+      .url_prefix = "/webhooks/",
+      .method = MHD_HTTP_METHOD_GET,
+      .have_id_segment = true,
+      .allow_deleted_instance = true,
+      .handler = &TMH_private_get_webhooks_ID
+    },
+    /* DELETE /webhooks/$ID/: */
+    {
+      .url_prefix = "/webhooks/",
+      .method = MHD_HTTP_METHOD_DELETE,
+      .have_id_segment = true,
+      .allow_deleted_instance = true,
+      .handler = &TMH_private_delete_webhooks_ID
+    },
+    /* PATCH /webhooks/$ID/: */
+    {
+      .url_prefix = "/webhooks/",
+      .method = MHD_HTTP_METHOD_PATCH,
+      .have_id_segment = true,
+      .allow_deleted_instance = true,
+      .handler = &TMH_private_patch_webhooks_ID,
+      /* allow webhook data of up to 8 MB, that should be plenty;
+         note that exceeding #GNUNET_MAX_MALLOC_CHECKED (40 MB)
+         would require further changes to the allocation logic
+         in the code... */
+      .max_upload = 1024 * 1024 * 8
+    },
     {
       .url_prefix = NULL
     }
@@ -930,6 +1077,15 @@ url_handler (void *cls,
       .method = MHD_HTTP_METHOD_GET,
       .mime_type = "text/html",
       .skip_instance = true,
+      .handler = &spa_redirect,
+      .response_code = MHD_HTTP_FOUND
+    },
+    {
+      .url_prefix = "/webui/",
+      .method = MHD_HTTP_METHOD_GET,
+      .mime_type = "text/html",
+      .skip_instance = true,
+      .have_id_segment = true,
       .handler = &TMH_return_spa,
       .response_code = MHD_HTTP_OK
     },
@@ -1046,6 +1202,15 @@ url_handler (void *cls,
       .have_id_segment = true,
       .handler = &TMH_return_static
     },
+    /* POST /templates/$ID: */
+    {
+      .url_prefix = "/templates/",
+      .method = MHD_HTTP_METHOD_POST,
+      .have_id_segment = true,
+      .allow_deleted_instance = true,
+      .handler = &TMH_post_using_templates_ID,
+      .max_upload = 1024 * 1024
+    },
     {
       .url_prefix = "*",
       .method = MHD_HTTP_METHOD_OPTIONS,
@@ -1268,7 +1433,7 @@ url_handler (void *cls,
       {
         prefix_strlen = slash - url + 1; /* includes both '/'-es if present! */
         infix_url = slash + 1;
-        slash = strchr (&infix_url[1], '/');
+        slash = strchr (infix_url, '/');
         if (NULL == slash)
         {
           /* the infix was the rest */
@@ -1739,14 +1904,12 @@ run (void *cls,
                       RFC_8959_PREFIX,
                       strlen (RFC_8959_PREFIX))) )
   {
-    char *tmp;
-
-    GNUNET_asprintf (&tmp,
-                     "%s%s",
-                     RFC_8959_PREFIX,
-                     TMH_default_auth);
-    GNUNET_free (TMH_default_auth);
-    TMH_default_auth = tmp;
+    fprintf (stderr,
+             "Authentication token does not start with `%s' prefix\n",
+             RFC_8959_PREFIX);
+    result = GNUNET_SYSERR;
+    GNUNET_SCHEDULER_shutdown ();
+    return;
   }
   cfg = config;
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
@@ -1792,7 +1955,8 @@ run (void *cls,
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
-  TMH_statics_init ();
+  /* /static/ is currently not used */
+  /* (void) TMH_statics_init (); */
   elen = TMH_EXCHANGES_init (config);
   if (GNUNET_SYSERR == elen)
   {
