@@ -238,6 +238,12 @@ struct Exchange
    */
   bool trusted;
 
+  /**
+   * true if this exchange did return to use the
+   * response from /wire.
+   */
+  bool have_wire;
+
 };
 
 
@@ -296,7 +302,6 @@ keys_mgmt_cb (void *cls,
  * the closure.
  *
  * @param cls the exchange
- *
  */
 static void
 retry_exchange (void *cls)
@@ -308,16 +313,27 @@ retry_exchange (void *cls)
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Connecting to exchange %s in retry_exchange()\n",
               exchange->url);
-  if (NULL != exchange->conn)
+  if (NULL == exchange->conn)
   {
-    TALER_EXCHANGE_disconnect (exchange->conn);
-    exchange->conn = NULL;
+    exchange->conn = TALER_EXCHANGE_connect (merchant_curl_ctx,
+                                             exchange->url,
+                                             &keys_mgmt_cb,
+                                             exchange,
+                                             TALER_EXCHANGE_OPTION_END);
+  }
+  else
+  {
+    struct GNUNET_TIME_Timestamp next;
+
+    next = TALER_EXCHANGE_check_keys_current (exchange->conn,
+                                              TALER_EXCHANGE_CKF_NONE);
+    if (! GNUNET_TIME_absolute_is_zero (next.abs_time))
+    {
+      exchange->retry_task = GNUNET_SCHEDULER_add_at (next.abs_time,
+                                                      &retry_exchange,
+                                                      exchange);
+    }
   }
-  exchange->conn = TALER_EXCHANGE_connect (merchant_curl_ctx,
-                                           exchange->url,
-                                           &keys_mgmt_cb,
-                                           exchange,
-                                           TALER_EXCHANGE_OPTION_END);
   /* Note: while the API spec says 'returns NULL on error', the implementation
      actually never returns NULL. */
   GNUNET_break (NULL != exchange->conn);
@@ -617,6 +633,13 @@ process_find_operations (struct Exchange *exchange)
         .http_status = MHD_HTTP_OK,
       };
 
+      if ( (NULL != fo->wire_method) &&
+           (! exchange->have_wire) )
+      {
+        /* We needed /wire, but didn't get it. That's not "200 OK". */
+        hr.http_status = MHD_HTTP_BAD_GATEWAY;
+        hr.ec = TALER_EC_MERCHANT_GENERIC_EXCHANGE_WIRE_REQUEST_FAILED;
+      }
       fo->fc (fo->fc_cls,
               &hr,
               exchange->conn,
@@ -666,6 +689,7 @@ handle_wire_data (void *cls,
   {
     struct TMH_EXCHANGES_FindOperation *fo;
 
+    exchange->have_wire = false;
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 "Failed to obtain /wire details from `%s': %u/%d\n",
                 exchange->url,
@@ -700,6 +724,7 @@ handle_wire_data (void *cls,
     };
 
     GNUNET_break_op (0);
+    exchange->have_wire = false;
     while (NULL != (fo = exchange->fo_head))
     {
       fo->fc (fo->fc_cls,
@@ -712,6 +737,7 @@ handle_wire_data (void *cls,
     }
     return;
   }
+  exchange->have_wire = true;
   if ( (process_find_operations (exchange)) &&
        (NULL == exchange->wire_task) &&
        (NULL == exchange->wire_request) )
@@ -999,9 +1025,16 @@ keys_mgmt_cb (void *cls,
   expire = TALER_EXCHANGE_check_keys_current (exchange->conn,
                                               TALER_EXCHANGE_CKF_NONE);
   if (0 == GNUNET_TIME_absolute_is_zero (expire.abs_time))
+  {
     delay = RELOAD_DELAY;
+  }
   else
+  {
     delay = GNUNET_TIME_absolute_get_remaining (expire.abs_time);
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "/keys response from expires at %s! Retrying at that time!\n",
+                GNUNET_TIME_absolute2s (expire.abs_time));
+  }
   if (GNUNET_TIME_relative_is_zero (delay))
   {
     GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -1338,12 +1371,21 @@ TMH_EXCHANGES_done ()
 {
   while (NULL != exchange_head)
     free_exchange_entry (exchange_head);
-  GNUNET_CURL_fini (merchant_curl_ctx);
-  merchant_curl_ctx = NULL;
-  GNUNET_CURL_gnunet_rc_destroy (merchant_curl_rc);
-  merchant_curl_rc = NULL;
-  json_decref (TMH_trusted_exchanges);
-  TMH_trusted_exchanges = NULL;
+  if (NULL != merchant_curl_ctx)
+  {
+    GNUNET_CURL_fini (merchant_curl_ctx);
+    merchant_curl_ctx = NULL;
+  }
+  if (NULL != merchant_curl_rc)
+  {
+    GNUNET_CURL_gnunet_rc_destroy (merchant_curl_rc);
+    merchant_curl_rc = NULL;
+  }
+  if (NULL != TMH_trusted_exchanges)
+  {
+    json_decref (TMH_trusted_exchanges);
+    TMH_trusted_exchanges = NULL;
+  }
 }
 
 
