@@ -23,6 +23,8 @@
 #include <taler/taler_util.h>
 #include <taler/taler_json_lib.h>
 #include "taler-merchant-httpd_helper.h"
+#include <taler/taler_templating_lib.h>
+
 
 /**
  * check @a payto_uris for well-formedness
@@ -271,8 +273,8 @@ TMH_products_array_valid (const json_t *products)
     if ( (NULL != taxes) &&
          (! TMH_taxes_array_valid (taxes)) )
     {
-        GNUNET_break_op (0);
-        valid = false;
+      GNUNET_break_op (0);
+      valid = false;
     }
     if ( (NULL != description_i18n) &&
          (! TALER_JSON_check_i18n (description_i18n)) )
@@ -317,6 +319,47 @@ TMH_image_data_url_valid (const char *image_data_url)
 }
 
 
+bool
+TMH_template_contract_valid (const json_t *template_contract)
+{
+  const char *summary;
+  struct TALER_Amount amount = { .value = 0};
+  uint32_t minimum_age = 0;
+  struct GNUNET_TIME_Relative pay_duration = { 0 };
+  struct GNUNET_JSON_Specification spec[] = {
+    GNUNET_JSON_spec_mark_optional (
+      GNUNET_JSON_spec_string ("summary",
+                               &summary),
+      NULL),
+    GNUNET_JSON_spec_mark_optional (
+      TALER_JSON_spec_amount ("amount",
+                              TMH_currency,
+                              &amount),
+      NULL),
+    GNUNET_JSON_spec_uint32 ("minimum_age",
+                             &minimum_age),
+    GNUNET_JSON_spec_relative_time ("pay_duration",
+                                    &pay_duration),
+    GNUNET_JSON_spec_end ()
+  };
+  const char *ename;
+  unsigned int eline;
+
+  if (GNUNET_OK !=
+      GNUNET_JSON_parse (template_contract,
+                         spec,
+                         &ename,
+                         &eline))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Invalid template_contract for field %s\n",
+                ename);
+    return false;
+  }
+  return true;
+}
+
+
 bool
 TMH_taxes_array_valid (const json_t *taxes)
 {
@@ -455,3 +498,125 @@ TMH_uuid_from_string (const char *uuids,
           &hc,
           sizeof (*uuid));
 }
+
+
+/**
+ * Closure for #trigger_webhook_cb.
+ *
+ * @param instance which is the instance we work with
+ * @param root JSON data to fill into the template
+ * @param rv, query of the TALER_TEMPLATEING_fill
+ */
+struct Trigger
+{
+  const char *instance;
+
+  const json_t *root;
+
+  enum GNUNET_DB_QueryStatus rv;
+
+};
+
+/**
+ * Typically called by `TMH_trigger_webhook`.
+ *
+ * @param[in,out] cls a `struct Trigger` with information about the webhook
+ * @param webhook_serial reference to the configured webhook template.
+ * @param event_type is the event/action of the webhook
+ * @param url to make request to
+ * @param http_method use for the webhook
+ * @param header_template of the webhook
+ * @param body_template of the webhook
+ */
+static void
+trigger_webhook_cb (void *cls,
+                    uint64_t webhook_serial,
+                    const char *event_type,
+                    const char *url,
+                    const char *http_method,
+                    const char *header_template,
+                    const char *body_template)
+{
+  struct Trigger *t = cls;
+  void *header = NULL;
+  void *body = NULL;
+  size_t header_size;
+  size_t body_size;
+
+  if (NULL != header_template)
+  {
+    int ret;
+
+    ret = TALER_TEMPLATING_fill (header_template,
+                                 t->root,
+                                 &header,
+                                 &header_size);
+    if (0 != ret)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                  "Failed to expand webhook header template for webhook %llu (%d)\n",
+                  (unsigned long long) webhook_serial,
+                  ret);
+      t->rv = GNUNET_DB_STATUS_HARD_ERROR;
+      return;
+    }
+    GNUNET_assert ('\0' == ((const char *) header)[header_size-1]);
+  }
+  if (NULL != body_template)
+  {
+    int ret;
+    ret = TALER_TEMPLATING_fill (body_template,
+                                 t->root,
+                                 &body,
+                                 &body_size);
+    if (0 != ret)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                  "Failed to expand webhook header template for webhook %llu (%d)\n",
+                  (unsigned long long) webhook_serial,
+                  ret);
+      t->rv = GNUNET_DB_STATUS_HARD_ERROR;
+      return;
+    }
+    GNUNET_assert ('\0' == ((const char *) body)[body_size-1]);
+  }
+  t->rv = TMH_db->insert_pending_webhook (TMH_db->cls,
+                                          t->instance,
+                                          webhook_serial,
+                                          url,
+                                          http_method,
+                                          header,
+                                          body);
+  free (header);
+  free (body);
+}
+
+
+/**
+ * TMH_trigger_webhook is a function that need to be use when someone
+ * pay. Merchant need to have a notification.
+ *
+ * @param instance that we need to send the webhook as a notification
+ * @param event of the webhook
+ * @param args argument of the function
+ */
+enum GNUNET_DB_QueryStatus
+TMH_trigger_webhook (const char *instance,
+                     const char *event,
+                     const json_t *args)
+{
+  struct Trigger t = {
+    .instance = instance,
+    .root = args
+  };
+  enum GNUNET_DB_QueryStatus qs;
+
+  qs = TMH_db->lookup_webhook_by_event (TMH_db->cls,
+                                        instance,
+                                        event,
+                                        &trigger_webhook_cb,
+                                        &t);
+  if (qs < 0)
+    return qs;
+  return t.rv;
+}
