@@ -87,6 +87,18 @@ bool
 TMH_image_data_url_valid (const char *image_data_url);
 
 
+/**
+ * Check if @a template_contract is a valid template_contract  object in the sense of Taler's API
+ * definition.
+ *
+ * @param template_contract object to check
+ * @return true if @a template_location is an object
+ *         representing a template_location.
+ */
+bool
+TMH_template_contract_valid (const json_t *template_contract);
+
+
 /**
  * Setup new wire method for the given @ payto_uri.
  *
@@ -99,6 +111,7 @@ TMH_setup_wire_account (const char *payto_uri);
 /**
  * FIXME: document
  */
+
 enum GNUNET_GenericReturnValue
 TMH_check_auth_config (struct MHD_Connection *connection,
                        const json_t *jauth,
@@ -130,4 +143,21 @@ TMH_uuid_from_string (const char *uuids,
   GNUNET_JSON_pack_allow_null ( \
     GNUNET_JSON_pack_object_incref ("exchange_reply", (json_t *) (hr)->reply))
 
+
+
+/**
+ * TMH_trigger_webhook is a function that need to be use when someone
+ * pay. Merchant need to have a notification.
+ *
+ * @param instance that we need to send the webhook as a notification
+ * @param event of the webhook
+ * @param args argument of the function
+ */
+enum GNUNET_DB_QueryStatus
+TMH_trigger_webhook (const char *instance,
+                     const char *action,
+                     const json_t *args);
+
+
+
 #endif
