@@ -126,7 +126,7 @@ struct AbortContext
   /**
    * Hashed contract terms (according to client).
    */
-  struct GNUNET_HashCode h_contract_terms;
+  struct TALER_PrivateContractHashP h_contract_terms;
 
   /**
    * Context for our operation.
@@ -658,13 +658,13 @@ refund_coins (void *cls,
               const struct TALER_Amount *wire_fee)
 {
   struct AbortContext *ac = cls;
-  struct GNUNET_TIME_Absolute now;
+  struct GNUNET_TIME_Timestamp now;
 
   (void) amount_with_fee;
   (void) deposit_fee;
   (void) refund_fee;
   (void) wire_fee;
-  now = GNUNET_TIME_absolute_get ();
+  now = GNUNET_TIME_timestamp_get ();
   for (unsigned int i = 0; i<ac->coins_cnt; i++)
   {
     struct RefundDetails *rdi = &ac->rd[i];
@@ -746,7 +746,7 @@ begin_transaction (struct AbortContext *ac)
   /* check payment was indeed incomplete
      (now that we are in the transaction scope!) */
   {
-    struct GNUNET_HashCode h_contract_terms;
+    struct TALER_PrivateContractHashP h_contract_terms;
     bool paid;
 
     qs = TMH_db->lookup_order_status (TMH_db->cls,
@@ -940,7 +940,7 @@ parse_abort (struct MHD_Connection *connection,
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Handling /abort for order `%s' with contract hash `%s'\n",
               ac->hc->infix,
-              GNUNET_h2s (&ac->h_contract_terms));
+              GNUNET_h2s (&ac->h_contract_terms.hash));
   return GNUNET_OK;
 }
 
