@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2020-2021 Taler Systems SA
+  (C) 2020-2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU Affero General Public License as
@@ -23,9 +23,11 @@
  * @author Jonathan Buchanan
  */
 #include "platform.h"
+#include <taler/taler_dbevents.h>
 #include <taler/taler_signatures.h>
 #include <taler/taler_json_lib.h>
 #include <taler/taler_exchange_service.h>
+#include "taler-merchant-httpd.h"
 #include "taler-merchant-httpd_auditors.h"
 #include "taler-merchant-httpd_exchanges.h"
 #include "taler-merchant-httpd_post-orders-ID-refund.h"
@@ -77,7 +79,7 @@ struct CoinRefund
    * When did the merchant grant the refund. To be used to group events
    * in the wallet.
    */
-  struct GNUNET_TIME_Absolute execution_time;
+  struct GNUNET_TIME_Timestamp execution_time;
 
   /**
    * Coin to refund.
@@ -132,7 +134,7 @@ struct PostRefundData
   /**
    * Hashed version of contract terms. All zeros if not provided.
    */
-  struct GNUNET_HashCode h_contract_terms;
+  struct TALER_PrivateContractHashP h_contract_terms;
 
   /**
    * DLL of (suspended) requests.
@@ -165,11 +167,6 @@ struct PostRefundData
    */
   struct TMH_SuspendedConnection sc;
 
-  /**
-   * Which merchant instance is this for?
-   */
-  struct MerchantInstance *mi;
-
   /**
    * order ID for the payment
    */
@@ -204,9 +201,13 @@ struct PostRefundData
   struct TALER_Amount refund_amount;
 
   /**
-   * Did we suspend @a connection?
+   * Did we suspend @a connection and are thus in
+   * the #prd_head DLL (#GNUNET_YES). Set to
+   * #GNUNET_NO if we are not suspended, and to
+   * #GNUNET_SYSERR if we should close the connection
+   * without a response due to shutdown.
    */
-  bool suspended;
+  enum GNUNET_GenericReturnValue suspended;
 
   /**
    * Return code: #TALER_EC_NONE if successful.
@@ -288,9 +289,7 @@ refund_cleanup (void *ctx)
     }
     GNUNET_free (cr);
   }
-
   json_decref (prd->contract_terms);
-
   GNUNET_free (prd);
 }
 
@@ -308,8 +307,8 @@ TMH_force_wallet_refund_order_resume (void)
     GNUNET_CONTAINER_DLL_remove (prd_head,
                                  prd_tail,
                                  prd);
-    GNUNET_assert (prd->suspended);
-    prd->suspended = false;
+    GNUNET_assert (GNUNET_YES == prd->suspended);
+    prd->suspended = GNUNET_SYSERR;
     MHD_resume_connection (prd->sc.con);
   }
 }
@@ -351,12 +350,37 @@ check_resume_prd (struct PostRefundData *prd)
                                prd_tail,
                                prd);
   GNUNET_assert (prd->suspended);
-  prd->suspended = false;
+  prd->suspended = GNUNET_NO;
   MHD_resume_connection (prd->sc.con);
   TALER_MHD_daemon_trigger ();
 }
 
 
+/**
+ * Notify applications waiting for a client to obtain
+ * a refund.
+ *
+ * @param prd refund request with the change
+ */
+static void
+notify_refund_obtained (struct PostRefundData *prd)
+{
+  struct TMH_OrderPayEventP refund_eh = {
+    .header.size = htons (sizeof (refund_eh)),
+    .header.type = htons (TALER_DBEVENT_MERCHANT_REFUND_OBTAINED),
+    .merchant_pub = prd->hc->instance->merchant_pub
+  };
+
+  GNUNET_CRYPTO_hash (prd->order_id,
+                      strlen (prd->order_id),
+                      &refund_eh.h_order_id);
+  TMH_db->event_notify (TMH_db->cls,
+                        &refund_eh.header,
+                        NULL,
+                        0);
+}
+
+
 /**
  * Callbacks of this type are used to serve the result of submitting a
  * refund request to an exchange.
@@ -403,6 +427,10 @@ refund_cb (void *cls,
                   "Failed to persist exchange response to /refund in database: %d\n",
                   qs);
     }
+    else
+    {
+      notify_refund_obtained (cr->prd);
+    }
   }
   check_resume_prd (cr->prd);
 }
@@ -476,7 +504,7 @@ exchange_found_cb (void *cls,
 static void
 process_refunds_cb (void *cls,
                     uint64_t refund_serial,
-                    struct GNUNET_TIME_Absolute timestamp,
+                    struct GNUNET_TIME_Timestamp timestamp,
                     const struct TALER_CoinSpendPublicKeyP *coin_pub,
                     const char *exchange_url,
                     uint64_t rtransaction_id,
@@ -568,12 +596,14 @@ TMH_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
     {
       json_t *contract_terms;
       uint64_t order_serial;
+      bool paid;
 
       qs = TMH_db->lookup_contract_terms (TMH_db->cls,
                                           hc->instance->settings.id,
                                           hc->infix,
                                           &contract_terms,
                                           &order_serial,
+                                          &paid,
                                           NULL);
       if (0 > qs)
       {
@@ -596,7 +626,8 @@ TMH_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
                                            hc->infix);
       }
       {
-        struct GNUNET_HashCode h_contract_terms;
+        struct TALER_PrivateContractHashP h_contract_terms;
+
         if (GNUNET_OK !=
             TALER_JSON_contract_hash (contract_terms,
                                       &h_contract_terms))
@@ -621,6 +652,8 @@ TMH_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
       }
     }
   }
+  if (GNUNET_SYSERR == prd->suspended)
+    return MHD_NO; /* we are in shutdown */
 
   if (TALER_EC_NONE != prd->ec)
   {
@@ -691,11 +724,6 @@ TMH_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
       if (NULL == cr->exchange_reply)
       {
         /* We need to talk to the exchange */
-        /* Notify clients waiting for the refund to be obtained. */
-        TMH_long_poll_resume (hc->infix,
-                              hc->instance,
-                              &prd->refund_amount,
-                              true);
         cr->fo = TMH_EXCHANGES_find_exchange (cr->exchange_url,
                                               NULL,
                                               GNUNET_NO,
@@ -713,9 +741,9 @@ TMH_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
   /* Check if there are still exchange operations pending */
   if (exchange_operations_pending (prd))
   {
-    if (! prd->suspended)
+    if (GNUNET_NO == prd->suspended)
     {
-      prd->suspended = true;
+      prd->suspended = GNUNET_YES;
       MHD_suspend_connection (connection);
       GNUNET_CONTAINER_DLL_insert (prd_head,
                                    prd_tail,
@@ -750,8 +778,8 @@ TMH_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
                                         &cr->coin_pub),
             TALER_JSON_pack_amount ("refund_amount",
                                     &cr->refund_amount),
-            GNUNET_JSON_pack_time_abs ("execution_time",
-                                       cr->execution_time));
+            GNUNET_JSON_pack_timestamp ("execution_time",
+                                        cr->execution_time));
         }
         else
         {
@@ -770,8 +798,8 @@ TMH_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
                                         &cr->coin_pub),
             TALER_JSON_pack_amount ("refund_amount",
                                     &cr->refund_amount),
-            GNUNET_JSON_pack_time_abs ("execution_time",
-                                       cr->execution_time));
+            GNUNET_JSON_pack_timestamp ("execution_time",
+                                        cr->execution_time));
         }
       }
       else
@@ -791,8 +819,8 @@ TMH_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
                                       &cr->coin_pub),
           TALER_JSON_pack_amount ("refund_amount",
                                   &cr->refund_amount),
-          GNUNET_JSON_pack_time_abs ("execution_time",
-                                     cr->execution_time));
+          GNUNET_JSON_pack_timestamp ("execution_time",
+                                      cr->execution_time));
       }
       GNUNET_assert (
         0 ==
