@@ -144,13 +144,13 @@ struct PickupContext
   /**
    * Which tip is being picked up?
    */
-  struct GNUNET_HashCode tip_id;
+  struct TALER_TipIdentifierP tip_id;
 
   /**
    * What is the ID of the pickup operation? (Basically a
    * hash over the key inputs).
    */
-  struct GNUNET_HashCode pickup_id;
+  struct TALER_PickupIdentifierP pickup_id;
 
   /**
    * Array of our planchets.
@@ -241,7 +241,7 @@ pick_context_cleanup (void *cls)
 
   stop_operations (pc); /* should not be any... */
   for (unsigned int i = 0; i<pc->planchets_length; i++)
-    GNUNET_free (pc->planchets[i].coin_ev);
+    TALER_planchet_detail_free (&pc->planchets[i]);
   GNUNET_array_grow (pc->planchets,
                      pc->planchets_length,
                      0);
@@ -249,17 +249,20 @@ pick_context_cleanup (void *cls)
 }
 
 
-/**
- * We are shutting down, force resuming all suspended pickup operations.
- */
 void
 TMH_force_tip_pickup_resume ()
 {
+  struct PickupContext *nxt;
+
   for (struct PickupContext *pc = pc_head;
        NULL != pc;
-       pc = pc->next)
+       pc = nxt)
   {
+    nxt = pc->next;
     stop_operations (pc);
+    GNUNET_CONTAINER_DLL_remove (pc_head,
+                                 pc_tail,
+                                 pc);
     MHD_resume_connection (pc->connection);
   }
 }
@@ -278,7 +281,7 @@ TMH_force_tip_pickup_resume ()
 static void
 withdraw_cb (void *cls,
              const struct TALER_EXCHANGE_HttpResponse *hr,
-             const struct GNUNET_CRYPTO_RsaSignature *blind_sig)
+             const struct TALER_BlindedDenominationSignature *blind_sig)
 {
   struct PlanchetOperation *po = cls;
   struct PickupContext *pc = po->pc;
@@ -287,6 +290,7 @@ withdraw_cb (void *cls,
   GNUNET_CONTAINER_DLL_remove (pc->po_head,
                                pc->po_tail,
                                po);
+  TMH_db->preflight (TMH_db->cls);
   if (NULL == blind_sig)
   {
     GNUNET_free (po);
@@ -296,6 +300,9 @@ withdraw_cb (void *cls,
       TALER_MHD_MAKE_JSON_PACK (
         TALER_JSON_pack_ec (TALER_EC_MERCHANT_TIP_PICKUP_EXCHANGE_ERROR),
         TMH_pack_exchange_reply (hr));
+    GNUNET_CONTAINER_DLL_remove (pc_head,
+                                 pc_tail,
+                                 pc);
     MHD_resume_connection (pc->connection);
     TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
     return;
@@ -312,6 +319,9 @@ withdraw_cb (void *cls,
     pc->response = TALER_MHD_make_error (
       TALER_EC_GENERIC_DB_STORE_FAILED,
       "blind signature");
+    GNUNET_CONTAINER_DLL_remove (pc_head,
+                                 pc_tail,
+                                 pc);
     MHD_resume_connection (pc->connection);
     TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
     return;
@@ -319,6 +329,9 @@ withdraw_cb (void *cls,
   if (NULL == pc->po_head)
   {
     stop_operations (pc); /* stops timeout job */
+    GNUNET_CONTAINER_DLL_remove (pc_head,
+                                 pc_tail,
+                                 pc);
     MHD_resume_connection (pc->connection);
     TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
   }
@@ -349,6 +362,7 @@ do_withdraw (void *cls,
   struct PickupContext *pc = po->pc;
 
   po->fo = NULL;
+  TMH_db->preflight (TMH_db->cls);
   if (NULL == hr)
   {
     stop_operations (pc);
@@ -409,6 +423,7 @@ try_withdraw (struct PickupContext *pc,
 {
   struct PlanchetOperation *po;
 
+  TMH_db->preflight (TMH_db->cls);
   po = GNUNET_new (struct PlanchetOperation);
   po->pc = pc;
   po->pd = *planchet;
@@ -441,6 +456,9 @@ do_timeout (void *cls)
   pc->response =  TALER_MHD_make_error (
     TALER_EC_MERCHANT_GENERIC_EXCHANGE_TIMEOUT,
     NULL);
+  GNUNET_CONTAINER_DLL_remove (pc_head,
+                               pc_tail,
+                               pc);
   MHD_resume_connection (pc->connection);
   TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
 }
@@ -594,15 +612,6 @@ reply_lookup_tip_failed (struct MHD_Connection *connection,
 }
 
 
-/**
- * Manages a POST /tips/$ID/pickup call, checking that the tip is authorized,
- * and if so, returning the blind signatures.
- *
- * @param rh context of the handler
- * @param connection the MHD connection to handle
- * @param[in,out] hc context with further information about the request
- * @return MHD result code
- */
 MHD_RESULT
 TMH_post_tips_ID_pickup (const struct TMH_RequestHandler *rh,
                          struct MHD_Connection *connection,
@@ -613,7 +622,7 @@ TMH_post_tips_ID_pickup (const struct TMH_RequestHandler *rh,
   struct TALER_Amount total_authorized;
   struct TALER_Amount total_picked_up;
   struct TALER_Amount total_remaining;
-  struct GNUNET_TIME_Absolute expiration;
+  struct GNUNET_TIME_Timestamp expiration;
   enum GNUNET_DB_QueryStatus qs;
   unsigned int num_retries;
 
@@ -630,7 +639,7 @@ TMH_post_tips_ID_pickup (const struct TMH_RequestHandler *rh,
     GNUNET_assert (NULL != hc->infix);
     if (GNUNET_OK !=
         GNUNET_CRYPTO_hash_from_string (hc->infix,
-                                        &pc->tip_id))
+                                        &pc->tip_id.hash))
     {
       /* tip_id has wrong encoding */
       GNUNET_break_op (0);
@@ -676,9 +685,8 @@ TMH_post_tips_ID_pickup (const struct TMH_RequestHandler *rh,
       struct GNUNET_JSON_Specification spec[] = {
         GNUNET_JSON_spec_fixed_auto ("denom_pub_hash",
                                      &pd->denom_pub_hash),
-        GNUNET_JSON_spec_varsize ("coin_ev",
-                                  (void **) &pd->coin_ev,
-                                  &pd->coin_ev_size),
+        TALER_JSON_spec_blinded_planchet ("coin_ev",
+                                          &pd->blinded_planchet),
         GNUNET_JSON_spec_end ()
       };
       {
@@ -711,12 +719,11 @@ TMH_post_tips_ID_pickup (const struct TMH_RequestHandler *rh,
         GNUNET_CRYPTO_hash_context_read (hc,
                                          &pd->denom_pub_hash,
                                          sizeof (pd->denom_pub_hash));
-        GNUNET_CRYPTO_hash_context_read (hc,
-                                         pd->coin_ev,
-                                         pd->coin_ev_size);
+        TALER_blinded_planchet_hash_ (&pd->blinded_planchet,
+                                      hc);
       }
       GNUNET_CRYPTO_hash_context_finish (hc,
-                                         &pc->pickup_id);
+                                         &pc->pickup_id.hash);
     }
   }
 
@@ -782,7 +789,8 @@ RETRY:
                                        NULL);
   }
   {
-    struct GNUNET_CRYPTO_RsaSignature *sigs[GNUNET_NZL (pc->planchets_length)];
+    struct TALER_BlindedDenominationSignature sigs[
+      GNUNET_NZL (pc->planchets_length)];
 
     memset (sigs,
             0,
@@ -795,6 +803,10 @@ RETRY:
                                 &pc->reserve_priv,
                                 pc->planchets_length,
                                 sigs);
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                "Lookup pickup `%s' resulted in %d\n",
+                GNUNET_h2s (&pc->pickup_id.hash),
+                qs);
     if (qs > GNUNET_DB_STATUS_SUCCESS_ONE_RESULT)
       qs = GNUNET_DB_STATUS_SUCCESS_ONE_RESULT;
     if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs)
@@ -803,7 +815,7 @@ RETRY:
 
       for (unsigned int i = 0; i< pc->planchets_length; i++)
       {
-        if (NULL != sigs[i])
+        if (TALER_DENOMINATION_INVALID != sigs[i].cipher)
           continue;
         if (! rollback)
         {
@@ -817,6 +829,10 @@ RETRY:
                                                  pc);
           rollback = true;
         }
+        GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                    "Lookup pickup `%s' initiated withdraw #%u\n",
+                    GNUNET_h2s (&pc->pickup_id.hash),
+                    i);
         try_withdraw (pc,
                       exchange_url,
                       &pc->planchets[i],
@@ -847,9 +863,9 @@ RETRY:
                            json_array_append_new (
                              blind_sigs,
                              GNUNET_JSON_PACK (
-                               GNUNET_JSON_pack_rsa_signature ("blind_sig",
-                                                               sigs[i]))));
-            GNUNET_CRYPTO_rsa_signature_free (sigs[i]);
+                               TALER_JSON_pack_blinded_denom_sig ("blind_sig",
+                                                                  &sigs[i]))));
+            TALER_blinded_denom_sig_free (&sigs[i]);
           }
           return TALER_MHD_REPLY_JSON_PACK (
             connection,
@@ -896,7 +912,7 @@ RETRY:
     return reply_lookup_tip_failed (connection,
                                     qs);
   }
-  if (GNUNET_TIME_absolute_is_past (expiration))
+  if (GNUNET_TIME_absolute_is_past (expiration.abs_time))
   {
     GNUNET_free (exchange_url);
     TMH_db->rollback (TMH_db->cls);
@@ -967,6 +983,9 @@ RETRY:
                                        NULL);
   }
   MHD_suspend_connection (connection);
+  GNUNET_CONTAINER_DLL_insert (pc_head,
+                               pc_tail,
+                               pc);
   pc->tt = GNUNET_SCHEDULER_add_delayed (EXCHANGE_TIMEOUT,
                                          &do_timeout,
                                          pc);
