@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2017-2021 Taler Systems SA
+  (C) 2017-2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU General Public License as published by the Free Software
@@ -60,6 +60,11 @@ struct TransferQuery
    */
   struct TransferQuery *prev;
 
+  /**
+   * Base URL of the exchange.
+   */
+  char *exchange_url;
+
   /**
    * Handle to query exchange about deposit status.
    */
@@ -78,7 +83,7 @@ struct TransferQuery
   /**
    * Hash of the merchant's bank account the transfer (presumably) went to.
    */
-  struct GNUNET_HashCode h_wire;
+  struct TALER_MerchantWireHashP h_wire;
 
   /**
    * Value deposited (including deposit fee).
@@ -158,10 +163,17 @@ struct GetOrderRequestContext
   struct GNUNET_SCHEDULER_Task *tt;
 
   /**
-   * Database event we are waiting on to be resuming.
+   * Database event we are waiting on to be resuming
+   * for payment or refunds.
    */
   struct GNUNET_DB_EventHandler *eh;
 
+  /**
+   * Database event we are waiting on to be resuming
+   * for session capture.
+   */
+  struct GNUNET_DB_EventHandler *session_eh;
+
   /**
    * Contract terms of the payment we are checking. NULL when they
    * are not (yet) known.
@@ -188,7 +200,7 @@ struct GetOrderRequestContext
   /**
    * Hash over the @e contract_terms.
    */
-  struct GNUNET_HashCode h_contract_terms;
+  struct TALER_PrivateContractHashP h_contract_terms;
 
   /**
    * Total amount the exchange deposited into our bank account
@@ -247,9 +259,13 @@ struct GetOrderRequestContext
   unsigned int wire_hc;
 
   /**
-   * Set to true if this request is currently suspended.
+   * Did we suspend @a connection and are thus in
+   * the #gorc_head DLL (#GNUNET_YES). Set to
+   * #GNUNET_NO if we are not suspended, and to
+   * #GNUNET_SYSERR if we should close the connection
+   * without a response due to shutdown.
    */
-  bool suspended;
+  enum GNUNET_GenericReturnValue suspended;
 
   /**
    * Set to true if this payment has been refunded and
@@ -283,6 +299,23 @@ static struct GetOrderRequestContext *gorc_head;
 static struct GetOrderRequestContext *gorc_tail;
 
 
+void
+TMH_force_gorc_resume (void)
+{
+  struct GetOrderRequestContext *gorc;
+
+  while (NULL != (gorc = gorc_head))
+  {
+    GNUNET_CONTAINER_DLL_remove (gorc_head,
+                                 gorc_tail,
+                                 gorc);
+    GNUNET_assert (GNUNET_YES == gorc->suspended);
+    gorc->suspended = GNUNET_SYSERR;
+    MHD_resume_connection (gorc->sc.con);
+  }
+}
+
+
 /**
  * Resume processing the request, cancelling all pending asynchronous
  * operations.
@@ -298,7 +331,6 @@ gorc_resume (struct GetOrderRequestContext *gorc,
 {
   struct TransferQuery *tq;
 
-  GNUNET_assert (gorc->suspended);
   if (NULL != gorc->tt)
   {
     GNUNET_SCHEDULER_cancel (gorc->tt);
@@ -319,10 +351,11 @@ gorc_resume (struct GetOrderRequestContext *gorc,
   }
   gorc->wire_hc = http_status;
   gorc->wire_ec = ec;
+  GNUNET_assert (GNUNET_YES == gorc->suspended);
   GNUNET_CONTAINER_DLL_remove (gorc_head,
                                gorc_tail,
                                gorc);
-  gorc->suspended = false;
+  gorc->suspended = GNUNET_NO;
   MHD_resume_connection (gorc->sc.con);
   TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
 }
@@ -345,10 +378,13 @@ resume_by_event (void *cls,
 
   (void) extra;
   (void) extra_size;
-  if (! gorc->suspended)
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Resuming request %p by trigger\n",
+              gorc);
+  if (GNUNET_NO == gorc->suspended)
     return; /* duplicate event is possible */
-  gorc->suspended = false;
-  GNUNET_CONTAINER_DLL_insert (gorc_head,
+  gorc->suspended = GNUNET_NO;
+  GNUNET_CONTAINER_DLL_remove (gorc_head,
                                gorc_tail,
                                gorc);
   MHD_resume_connection (gorc->sc.con);
@@ -412,13 +448,11 @@ exchange_timeout_cb (void *cls)
  * Function called with detailed wire transfer data.
  *
  * @param cls closure with a `struct TransferQuery *`
- * @param hr HTTP response data
- * @param dd details about the deposit (NULL on errors)
+ * @param dr HTTP response data
  */
 static void
 deposit_get_cb (void *cls,
-                const struct TALER_EXCHANGE_HttpResponse *hr,
-                const struct TALER_EXCHANGE_DepositData *dd)
+                const struct TALER_EXCHANGE_GetDepositResponse *dr)
 {
   struct TransferQuery *tq = cls;
   struct GetOrderRequestContext *gorc = tq->gorc;
@@ -426,32 +460,111 @@ deposit_get_cb (void *cls,
   GNUNET_CONTAINER_DLL_remove (gorc->tq_head,
                                gorc->tq_tail,
                                tq);
-  if (NULL == dd)
+  switch (dr->hr.http_status)
   {
-    gorc_report (gorc,
-                 TALER_EC_MERCHANT_GET_ORDERS_EXCHANGE_TRACKING_FAILURE,
-                 &tq->coin_pub,
-                 hr);
-    GNUNET_free (tq);
-    if (NULL == gorc->tq_head)
-      gorc_resume (gorc,
-                   0,
-                   TALER_EC_NONE);
-    return;
-  }
-  else if (MHD_HTTP_OK == hr->http_status)
-  {
-    enum GNUNET_DB_QueryStatus qs;
+  case MHD_HTTP_OK:
+    {
+      enum GNUNET_DB_QueryStatus qs;
 
-    qs = TMH_db->insert_deposit_to_transfer (TMH_db->cls,
-                                             tq->deposit_serial,
-                                             dd);
-    if (qs < 0)
+      qs = TMH_db->insert_deposit_to_transfer (TMH_db->cls,
+                                               tq->deposit_serial,
+                                               &dr->details.success);
+      if (qs < 0)
+      {
+        gorc_report (gorc,
+                     TALER_EC_GENERIC_DB_STORE_FAILED,
+                     &tq->coin_pub,
+                     NULL);
+        GNUNET_free (tq->exchange_url);
+        GNUNET_free (tq);
+        if (NULL == gorc->tq_head)
+          gorc_resume (gorc,
+                       0,
+                       TALER_EC_NONE);
+        return;
+      }
+      /* Compute total amount *wired* */
+      if (0 >
+          TALER_amount_add (&gorc->deposits_total,
+                            &gorc->deposits_total,
+                            &dr->details.success.coin_contribution))
+      {
+        gorc_report (gorc,
+                     TALER_EC_MERCHANT_PRIVATE_GET_ORDERS_ID_AMOUNT_ARITHMETIC_FAILURE,
+                     &tq->coin_pub,
+                     NULL);
+        GNUNET_free (tq->exchange_url);
+        GNUNET_free (tq);
+        if (NULL == gorc->tq_head)
+          gorc_resume (gorc,
+                       0,
+                       TALER_EC_NONE);
+        return;
+      }
+      if (0 >
+          TALER_amount_add (&gorc->deposit_fees_total,
+                            &gorc->deposit_fees_total,
+                            &tq->deposit_fee))
+      {
+        gorc_report (gorc,
+                     TALER_EC_MERCHANT_PRIVATE_GET_ORDERS_ID_AMOUNT_ARITHMETIC_FAILURE,
+                     &tq->coin_pub,
+                     NULL);
+        GNUNET_free (tq->exchange_url);
+        GNUNET_free (tq);
+        if (NULL == gorc->tq_head)
+          gorc_resume (gorc,
+                       0,
+                       TALER_EC_NONE);
+        return;
+      }
+      break;
+    }
+  case MHD_HTTP_ACCEPTED:
+    {
+      /* got a 'preliminary' reply from the exchange,
+         remember our target UUID */
+      enum GNUNET_DB_QueryStatus qs;
+      struct GNUNET_TIME_Timestamp now;
+
+      now = GNUNET_TIME_timestamp_get ();
+      qs = TMH_db->account_kyc_set_status (
+        TMH_db->cls,
+        gorc->hc->instance->settings.id,
+        &tq->h_wire,
+        tq->exchange_url,
+        dr->details.accepted.requirement_row,
+        NULL,
+        NULL,
+        now,
+        false);
+      if (qs < 0)
+      {
+        gorc_report (gorc,
+                     TALER_EC_GENERIC_DB_STORE_FAILED,
+                     &tq->coin_pub,
+                     NULL);
+        GNUNET_free (tq->exchange_url);
+        GNUNET_free (tq);
+        if (NULL == gorc->tq_head)
+          gorc_resume (gorc,
+                       0,
+                       TALER_EC_NONE);
+        return;
+      }
+      gorc_report (gorc,
+                   TALER_EC_NONE,
+                   &tq->coin_pub,
+                   &dr->hr);
+      break;
+    }
+  default:
     {
       gorc_report (gorc,
-                   TALER_EC_GENERIC_DB_STORE_FAILED,
+                   TALER_EC_MERCHANT_GET_ORDERS_EXCHANGE_TRACKING_FAILURE,
                    &tq->coin_pub,
-                   NULL);
+                   &dr->hr);
+      GNUNET_free (tq->exchange_url);
       GNUNET_free (tq);
       if (NULL == gorc->tq_head)
         gorc_resume (gorc,
@@ -459,48 +572,8 @@ deposit_get_cb (void *cls,
                      TALER_EC_NONE);
       return;
     }
-    /* Compute total amount *wired* */
-    if (0 >
-        TALER_amount_add (&gorc->deposits_total,
-                          &gorc->deposits_total,
-                          &dd->coin_contribution))
-    {
-      gorc_report (gorc,
-                   TALER_EC_MERCHANT_PRIVATE_GET_ORDERS_ID_AMOUNT_ARITHMETIC_FAILURE,
-                   &tq->coin_pub,
-                   NULL);
-      GNUNET_free (tq);
-      if (NULL == gorc->tq_head)
-        gorc_resume (gorc,
-                     0,
-                     TALER_EC_NONE);
-      return;
-    }
-    if (0 >
-        TALER_amount_add (&gorc->deposit_fees_total,
-                          &gorc->deposit_fees_total,
-                          &tq->deposit_fee))
-    {
-      gorc_report (gorc,
-                   TALER_EC_MERCHANT_PRIVATE_GET_ORDERS_ID_AMOUNT_ARITHMETIC_FAILURE,
-                   &tq->coin_pub,
-                   NULL);
-      GNUNET_free (tq);
-      if (NULL == gorc->tq_head)
-        gorc_resume (gorc,
-                     0,
-                     TALER_EC_NONE);
-      return;
-    }
-  }
-  else
-  {
-    /* got a 'preliminary' reply from the exchange, simply skip */
-    gorc_report (gorc,
-                 TALER_EC_NONE,
-                 &tq->coin_pub,
-                 hr);
-  }
+  } /* end switch */
+  GNUNET_free (tq->exchange_url);
   GNUNET_free (tq);
   if (NULL != gorc->tq_head)
     return;
@@ -540,6 +613,7 @@ exchange_found_cb (void *cls,
     GNUNET_CONTAINER_DLL_remove (gorc->tq_head,
                                  gorc->tq_tail,
                                  tq);
+    GNUNET_free (tq->exchange_url);
     GNUNET_free (tq);
     gorc_resume (gorc,
                  MHD_HTTP_GATEWAY_TIMEOUT,
@@ -552,6 +626,7 @@ exchange_found_cb (void *cls,
     GNUNET_CONTAINER_DLL_remove (gorc->tq_head,
                                  gorc->tq_tail,
                                  tq);
+    GNUNET_free (tq->exchange_url);
     GNUNET_free (tq);
     gorc->exchange_hc = hr->http_status;
     gorc->exchange_ec = hr->ec;
@@ -572,6 +647,7 @@ exchange_found_cb (void *cls,
     GNUNET_CONTAINER_DLL_remove (gorc->tq_head,
                                  gorc->tq_tail,
                                  tq);
+    GNUNET_free (tq->exchange_url);
     GNUNET_free (tq);
     gorc_resume (gorc,
                  MHD_HTTP_INTERNAL_SERVER_ERROR,
@@ -600,7 +676,7 @@ static void
 deposit_cb (void *cls,
             uint64_t deposit_serial,
             const char *exchange_url,
-            const struct GNUNET_HashCode *h_wire,
+            const struct TALER_MerchantWireHashP *h_wire,
             const struct TALER_Amount *amount_with_fee,
             const struct TALER_Amount *deposit_fee,
             const struct TALER_CoinSpendPublicKeyP *coin_pub)
@@ -610,6 +686,7 @@ deposit_cb (void *cls,
 
   tq = GNUNET_new (struct TransferQuery);
   tq->gorc = gorc;
+  tq->exchange_url = GNUNET_strdup (exchange_url);
   tq->deposit_serial = deposit_serial;
   GNUNET_CONTAINER_DLL_insert (gorc->tq_head,
                                gorc->tq_tail,
@@ -656,6 +733,11 @@ gorc_cleanup (void *cls)
     TMH_db->event_listen_cancel (gorc->eh);
     gorc->eh = NULL;
   }
+  if (NULL != gorc->session_eh)
+  {
+    TMH_db->event_listen_cancel (gorc->session_eh);
+    gorc->session_eh = NULL;
+  }
   GNUNET_free (gorc);
 }
 
@@ -677,7 +759,7 @@ gorc_cleanup (void *cls)
 static void
 process_refunds_cb (void *cls,
                     uint64_t refund_serial,
-                    struct GNUNET_TIME_Absolute timestamp,
+                    struct GNUNET_TIME_Timestamp timestamp,
                     const struct TALER_CoinSpendPublicKeyP *coin_pub,
                     const char *exchange_url,
                     uint64_t rtransaction_id,
@@ -693,8 +775,10 @@ process_refunds_cb (void *cls,
                    GNUNET_JSON_PACK (
                      TALER_JSON_pack_amount ("amount",
                                              refund_amount),
-                     GNUNET_JSON_pack_time_abs ("timestamp",
-                                                timestamp),
+                     GNUNET_JSON_pack_bool ("pending",
+                                            pending),
+                     GNUNET_JSON_pack_timestamp ("timestamp",
+                                                 timestamp),
                      GNUNET_JSON_pack_string ("reason",
                                               reason))));
   /* For refunded coins, we are not charged deposit fees, so subtract those
@@ -718,7 +802,7 @@ process_refunds_cb (void *cls,
                                    &gorc->refund_amount,
                                    refund_amount));
   gorc->refunded = true;
-  gorc->refund_pending = pending;
+  gorc->refund_pending |= pending;
 }
 
 
@@ -739,7 +823,7 @@ static void
 process_transfer_details (void *cls,
                           const struct TALER_WireTransferIdentifierRawP *wtid,
                           const char *exchange_url,
-                          struct GNUNET_TIME_Absolute execution_time,
+                          struct GNUNET_TIME_Timestamp execution_time,
                           const struct TALER_Amount *deposit_value,
                           const struct TALER_Amount *deposit_fee,
                           bool transfer_confirmed)
@@ -747,7 +831,6 @@ process_transfer_details (void *cls,
   struct GetOrderRequestContext *gorc = cls;
   json_t *wire_details = gorc->wire_details;
   struct TALER_Amount wired;
-  struct GNUNET_TIME_Absolute execution_time_round = execution_time;
 
   /* Compute total amount *wired* */
   GNUNET_assert (0 <
@@ -759,7 +842,6 @@ process_transfer_details (void *cls,
                                    &gorc->deposit_fees_total,
                                    deposit_fee));
 
-  GNUNET_TIME_round_abs (&execution_time_round);
   GNUNET_assert
     (0 <= TALER_amount_subtract (&wired,
                                  deposit_value,
@@ -774,8 +856,8 @@ process_transfer_details (void *cls,
                                               exchange_url),
                      TALER_JSON_pack_amount ("amount",
                                              &wired),
-                     GNUNET_JSON_pack_time_abs ("execution_time",
-                                                execution_time_round),
+                     GNUNET_JSON_pack_timestamp ("execution_time",
+                                                 execution_time),
                      GNUNET_JSON_pack_bool ("confirmed",
                                             transfer_confirmed))));
 }
@@ -793,7 +875,7 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
   bool order_only = false;
   struct TALER_ClaimTokenP claim_token = { 0 };
   const char *summary;
-  struct GNUNET_TIME_Absolute timestamp;
+  struct GNUNET_TIME_Timestamp timestamp;
 
   if (NULL == gorc)
   {
@@ -857,19 +939,47 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
           = GNUNET_TIME_relative_to_absolute (timeout);
         if (! GNUNET_TIME_relative_is_zero (timeout))
         {
-          struct TMH_OrderPayEvent pay_eh = {
+          struct TMH_OrderPayEventP pay_eh = {
             .header.size = htons (sizeof (pay_eh)),
-            .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_PAID)
+            .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_PAID),
+            .merchant_pub = hc->instance->merchant_pub
           };
 
           GNUNET_CRYPTO_hash (hc->infix,
                               strlen (hc->infix),
                               &pay_eh.h_order_id);
+          GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                      "Subscribing to payment triggers for %p\n",
+                      gorc);
           gorc->eh = TMH_db->event_listen (TMH_db->cls,
                                            &pay_eh.header,
                                            timeout,
                                            &resume_by_event,
                                            gorc);
+          if ( (NULL != gorc->session_id) &&
+               (NULL != gorc->fulfillment_url) )
+          {
+            struct TMH_SessionEventP session_eh = {
+              .header.size = htons (sizeof (session_eh)),
+              .header.type = htons (TALER_DBEVENT_MERCHANT_SESSION_CAPTURED),
+              .merchant_pub = hc->instance->merchant_pub
+            };
+
+            GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                        "Subscribing to session triggers for %p\n",
+                        gorc);
+            GNUNET_CRYPTO_hash (gorc->session_id,
+                                strlen (gorc->session_id),
+                                &session_eh.h_session_id);
+            GNUNET_CRYPTO_hash (gorc->fulfillment_url,
+                                strlen (gorc->fulfillment_url),
+                                &session_eh.h_fulfillment_url);
+            gorc->session_eh = TMH_db->event_listen (TMH_db->cls,
+                                                     &session_eh.header,
+                                                     timeout,
+                                                     &resume_by_event,
+                                                     gorc);
+          }
         }
       }
       else
@@ -877,11 +987,12 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
         gorc->sc.long_poll_timeout = GNUNET_TIME_UNIT_ZERO_ABS;
       }
     }
-
-
   } /* end first-time per-request initialization */
 
-  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+  if (GNUNET_SYSERR == gorc->suspended)
+    return MHD_NO; /* we are in shutdown */
+
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Starting GET /private/orders/%s processing with timeout %s\n",
               hc->infix,
               GNUNET_STRINGS_absolute_time_to_string (
@@ -894,18 +1005,23 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
        earlier *unclaimed* order state (which we loaded in a previous
        invocation of this function and we are back here due to long polling)
        and thus the contract terms could have changed during claiming. Thus,
-       we need to fetch the latest contract terms from the DB again. *///
+       we need to fetch the latest contract terms from the DB again. */
     json_decref (gorc->contract_terms);
     gorc->contract_terms = NULL;
     gorc->fulfillment_url = NULL;
   }
   TMH_db->preflight (TMH_db->cls);
-  qs = TMH_db->lookup_contract_terms (TMH_db->cls,
-                                      hc->instance->settings.id,
-                                      hc->infix,
-                                      &gorc->contract_terms,
-                                      &gorc->order_serial,
-                                      NULL);
+  {
+    bool paid = false;
+
+    qs = TMH_db->lookup_contract_terms (TMH_db->cls,
+                                        hc->instance->settings.id,
+                                        hc->infix,
+                                        &gorc->contract_terms,
+                                        &gorc->order_serial,
+                                        &paid,
+                                        NULL);
+  }
   if (GNUNET_DB_STATUS_SUCCESS_NO_RESULTS == qs)
   {
     order_only = true;
@@ -924,7 +1040,7 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
   }
 
   {
-    struct GNUNET_HashCode unused;
+    struct TALER_MerchantPostDataHashP unused;
     json_t *ct = NULL;
 
     /* We need the order for two cases:  Either when the contract doesn't exist yet,
@@ -973,11 +1089,12 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
                               &gorc->contract_amount),
       GNUNET_JSON_spec_mark_optional (
         GNUNET_JSON_spec_string ("fulfillment_url",
-                                 &gorc->fulfillment_url)),
+                                 &gorc->fulfillment_url),
+        NULL),
       GNUNET_JSON_spec_string ("summary",
                                &summary),
-      TALER_JSON_spec_absolute_time ("timestamp",
-                                     &timestamp),
+      GNUNET_JSON_spec_timestamp ("timestamp",
+                                  &timestamp),
       GNUNET_JSON_spec_end ()
     };
 
@@ -1119,8 +1236,8 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
                                 &gorc->contract_amount),
         GNUNET_JSON_pack_string ("summary",
                                  summary),
-        GNUNET_JSON_pack_time_abs ("creation_time",
-                                   timestamp));
+        GNUNET_JSON_pack_timestamp ("creation_time",
+                                    timestamp));
       GNUNET_free (taler_pay_uri);
       GNUNET_free (already_paid_order_id);
       return ret;
@@ -1135,16 +1252,14 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
   {
     if (GNUNET_TIME_absolute_is_future (gorc->sc.long_poll_timeout))
     {
-      GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+      GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                   "Suspending GET /private/orders/%s\n",
                   hc->infix);
-      gorc->suspended = true;
-      TMH_long_poll_suspend (hc->infix,
-                             gorc->session_id,
-                             gorc->fulfillment_url,
-                             hc->instance,
-                             &gorc->sc,
-                             NULL);
+      GNUNET_CONTAINER_DLL_insert (gorc_head,
+                                   gorc_tail,
+                                   gorc);
+      gorc->suspended = GNUNET_YES;
+      MHD_suspend_connection (gorc->sc.con);
       return MHD_YES;
     }
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
@@ -1179,11 +1294,11 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
       GNUNET_CONTAINER_DLL_insert (gorc_head,
                                    gorc_tail,
                                    gorc);
+      gorc->suspended = GNUNET_YES;
+      MHD_suspend_connection (connection);
       gorc->tt = GNUNET_SCHEDULER_add_delayed (EXCHANGE_TIMEOUT,
                                                &exchange_timeout_cb,
                                                gorc);
-      gorc->suspended = true;
-      MHD_suspend_connection (connection);
       return MHD_YES;
     }
   }
@@ -1191,16 +1306,15 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
   if ( (! paid) &&
        (GNUNET_TIME_absolute_is_future (gorc->sc.long_poll_timeout)) )
   {
-    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Suspending GET /private/orders/%s\n",
                 hc->infix);
-    gorc->suspended = true;
-    TMH_long_poll_suspend (hc->infix,
-                           gorc->session_id,
-                           gorc->fulfillment_url,
-                           hc->instance,
-                           &gorc->sc,
-                           NULL);
+    GNUNET_assert (GNUNET_NO == gorc->suspended);
+    GNUNET_CONTAINER_DLL_insert (gorc_head,
+                                 gorc_tail,
+                                 gorc);
+    gorc->suspended = GNUNET_YES;
+    MHD_suspend_connection (gorc->sc.con);
     return MHD_YES;
   }
 
@@ -1235,8 +1349,8 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
                               &gorc->contract_amount),
       GNUNET_JSON_pack_string ("summary",
                                summary),
-      GNUNET_JSON_pack_time_abs ("creation_time",
-                                 timestamp));
+      GNUNET_JSON_pack_timestamp ("creation_time",
+                                  timestamp));
     GNUNET_free (taler_pay_uri);
     GNUNET_free (order_status_url);
     return ret;
@@ -1329,17 +1443,15 @@ TMH_private_get_orders_ID (const struct TMH_RequestHandler *rh,
                                        gorc->order_serial);
         GNUNET_break (qs >= 0); /* just warn if transaction failed */
         TMH_notify_order_change (hc->instance,
-                                 hc->infix,
-                                 true, /* paid */
-                                 false, /* technically unknown, but OK here */
-                                 true, /* wired */
+                                 TMH_OSF_PAID
+                                 | TMH_OSF_WIRED,
                                  timestamp,
                                  gorc->order_serial);
       }
     }
 
     {
-      struct GNUNET_HashCode *h_contract = NULL;
+      struct TALER_PrivateContractHashP *h_contract = NULL;
 
       /* In a session-bound payment, allow the browser to check the order
        * status page (e.g. to get a refund).
