@@ -33,11 +33,11 @@ TMH_private_get_tips_ID (const struct TMH_RequestHandler *rh,
                          struct MHD_Connection *connection,
                          struct TMH_HandlerContext *hc)
 {
-  struct GNUNET_HashCode tip_id;
+  struct TALER_TipIdentifierP tip_id;
   struct TALER_Amount total_authorized;
   struct TALER_Amount total_picked_up;
   char *reason;
-  struct GNUNET_TIME_Absolute expiration;
+  struct GNUNET_TIME_Timestamp expiration;
   struct TALER_ReservePublicKeyP reserve_pub;
   unsigned int pickups_length = 0;
   struct TALER_MERCHANTDB_PickupDetails *pickups = NULL;
@@ -45,10 +45,12 @@ TMH_private_get_tips_ID (const struct TMH_RequestHandler *rh,
   bool fpu;
   json_t *pickups_json = NULL;
 
+  (void) rh;
   GNUNET_assert (NULL != hc->infix);
+  // FIXME: support long-polling (client-side already implemented)
   if (GNUNET_OK !=
       GNUNET_CRYPTO_hash_from_string (hc->infix,
-                                      &tip_id))
+                                      &tip_id.hash))
   {
     /* tip_id has wrong encoding */
     GNUNET_break_op (0);
@@ -131,11 +133,8 @@ TMH_private_get_tips_ID (const struct TMH_RequestHandler *rh,
                      pickups_length,
                      0);
   {
-    struct GNUNET_TIME_Absolute expiration_round = expiration;
     MHD_RESULT ret;
 
-    GNUNET_TIME_round_abs (&expiration_round);
-
     ret = TALER_MHD_REPLY_JSON_PACK (
       connection,
       MHD_HTTP_OK,
@@ -145,8 +144,8 @@ TMH_private_get_tips_ID (const struct TMH_RequestHandler *rh,
                               &total_picked_up),
       GNUNET_JSON_pack_string ("reason",
                                reason),
-      GNUNET_JSON_pack_time_abs ("expiration",
-                                 expiration_round),
+      GNUNET_JSON_pack_timestamp ("expiration",
+                                  expiration),
       GNUNET_JSON_pack_data_auto ("reserve_pub",
                                   &reserve_pub),
       GNUNET_JSON_pack_allow_null (
