@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014-2020 Taler Systems SA
+  (C) 2014-2021 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -21,6 +21,7 @@
  */
 #include "platform.h"
 #include <jansson.h>
+#include <taler/taler_dbevents.h>
 #include <taler/taler_signatures.h>
 #include <taler/taler_json_lib.h>
 #include "taler-merchant-httpd_private-post-orders-ID-refund.h"
@@ -34,6 +35,40 @@
 #define MAX_RETRIES 5
 
 
+/**
+ * Use database to notify other clients about the
+ * @a order_id being refunded
+ *
+ * @param hc handler context we operate in
+ * @param amount the (total) refunded amount
+ */
+static void
+trigger_refund_notification (struct TMH_HandlerContext *hc,
+                             const struct TALER_Amount *amount)
+{
+  const char *as;
+  struct TMH_OrderRefundEventP refund_eh = {
+    .header.size = htons (sizeof (refund_eh)),
+    .header.type = htons (TALER_DBEVENT_MERCHANT_ORDER_REFUND),
+    .merchant_pub = hc->instance->merchant_pub
+  };
+
+  /* Resume clients that may wait for this refund */
+  as = TALER_amount2s (amount);
+  GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+              "Awakening clients on %s waiting for refund of no more than %s\n",
+              hc->infix,
+              as);
+  GNUNET_CRYPTO_hash (hc->infix,
+                      strlen (hc->infix),
+                      &refund_eh.h_order_id);
+  TMH_db->event_notify (TMH_db->cls,
+                        &refund_eh.header,
+                        as,
+                        strlen (as));
+}
+
+
 /**
  * Make a taler://refund URI
  *
@@ -54,7 +89,6 @@ make_taler_refund_uri (struct MHD_Connection *connection,
 
   GNUNET_assert (NULL != instance_id);
   GNUNET_assert (NULL != order_id);
-
   host = MHD_lookup_connection_value (connection,
                                       MHD_HEADER_KIND,
                                       "Host");
@@ -86,7 +120,7 @@ make_taler_refund_uri (struct MHD_Connection *connection,
   }
   GNUNET_buffer_write_path (&buf, order_id);
   GNUNET_buffer_write_path (&buf,
-                            ""); // Trailing slash
+                            ""); /* Trailing slash */
   return GNUNET_buffer_reap_str (&buf);
 }
 
@@ -116,28 +150,30 @@ TMH_private_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
     GNUNET_JSON_spec_end ()
   };
   enum TALER_MERCHANTDB_RefundStatus rs;
-  struct GNUNET_HashCode h_contract;
+  struct TALER_PrivateContractHashP h_contract;
 
   {
     enum GNUNET_DB_QueryStatus qs;
     json_t *contract_terms;
     uint64_t order_serial;
-    struct GNUNET_TIME_Absolute refund_deadline;
-    struct GNUNET_TIME_Absolute timestamp;
+    struct GNUNET_TIME_Timestamp refund_deadline;
+    struct GNUNET_TIME_Timestamp timestamp;
+    bool paid = false;
 
     qs = TMH_db->lookup_contract_terms (TMH_db->cls,
                                         hc->instance->settings.id,
                                         hc->infix,
                                         &contract_terms,
                                         &order_serial,
+                                        &paid,
                                         NULL);
     if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT == qs)
     {
       struct GNUNET_JSON_Specification spec[] = {
-        TALER_JSON_spec_absolute_time ("refund_deadline",
-                                       &refund_deadline),
-        TALER_JSON_spec_absolute_time ("timestamp",
-                                       &timestamp),
+        GNUNET_JSON_spec_timestamp ("refund_deadline",
+                                    &refund_deadline),
+        GNUNET_JSON_spec_timestamp ("timestamp",
+                                    &timestamp),
         GNUNET_JSON_spec_end ()
       };
 
@@ -156,7 +192,9 @@ TMH_private_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
           "mandatory fields missing");
       }
       json_decref (contract_terms);
-      if (timestamp.abs_value_us == refund_deadline.abs_value_us)
+      if (GNUNET_TIME_timestamp_cmp (timestamp,
+                                     ==,
+                                     refund_deadline))
       {
         /* refund was never allowed, so we should refuse hard */
         return TALER_MHD_reply_with_error (
@@ -165,7 +203,7 @@ TMH_private_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
           TALER_EC_MERCHANT_PRIVATE_POST_ORDERS_ID_REFUND_NOT_ALLOWED_BY_CONTRACT,
           NULL);
       }
-      if (GNUNET_TIME_absolute_is_past (refund_deadline))
+      if (GNUNET_TIME_absolute_is_past (refund_deadline.abs_time))
       {
         /* it is too late for refunds */
         /* NOTE: We MAY still be lucky that the exchange did not yet
@@ -231,6 +269,8 @@ TMH_private_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
       }
       if (GNUNET_DB_STATUS_SOFT_ERROR == qs)
         continue;
+      trigger_refund_notification (hc,
+                                   &refund);
     }
     break;
   } /* retries loop */
@@ -268,12 +308,14 @@ TMH_private_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
       enum GNUNET_DB_QueryStatus qs;
       json_t *contract_terms;
       uint64_t order_serial;
+      bool paid;
 
       qs = TMH_db->lookup_contract_terms (TMH_db->cls,
                                           hc->instance->settings.id,
                                           hc->infix,
                                           &contract_terms,
                                           &order_serial,
+                                          &paid,
                                           NULL);
       if (GNUNET_DB_STATUS_SUCCESS_ONE_RESULT != qs)
       {
@@ -298,17 +340,8 @@ TMH_private_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
     break;
   }
 
-  /* Resume clients that may wait for this refund */
-  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Awakeing clients on %s waiting for refund of less than %s\n",
-              hc->infix,
-              TALER_amount2s (&refund));
-  TMH_long_poll_resume (hc->infix,
-                        hc->instance,
-                        &refund,
-                        false);
   {
-    struct GNUNET_TIME_Absolute timestamp;
+    struct GNUNET_TIME_Timestamp timestamp;
     uint64_t order_serial;
     enum GNUNET_DB_QueryStatus qs;
 
@@ -326,10 +359,9 @@ TMH_private_post_orders_ID_refund (const struct TMH_RequestHandler *rh,
                                          NULL);
     }
     TMH_notify_order_change (hc->instance,
-                             hc->infix,
-                             true, /* paid */
-                             true, /* refunded */
-                             false, /* wired, cannot be if we could still do refunds */
+                             TMH_OSF_CLAIMED
+                             | TMH_OSF_PAID
+                             | TMH_OSF_REFUNDED,
                              timestamp,
                              order_serial);
   }
