@@ -71,14 +71,48 @@ check_products (const json_t *products)
   }
   json_array_foreach (products, index, value) {
     const char *description;
+    const char *product_id = NULL;
+    uint64_t quantity;
+    const char *unit = NULL;
+    struct TALER_Amount price;
+    const char *image = NULL;
+    json_t *taxes = NULL;
+    struct GNUNET_TIME_Timestamp delivery_date;
     const char *error_name;
     unsigned int error_line;
     enum GNUNET_GenericReturnValue res;
     struct GNUNET_JSON_Specification spec[] = {
-      // FIXME: parse and format-validate all
-      // optional fields of a product and check validity
-      GNUNET_JSON_spec_string ("description",
-                               &description),
+      GNUNET_JSON_spec_mark_optional (
+        GNUNET_JSON_spec_string ("product_id",
+                                 &product_id),
+        NULL),
+      TALER_JSON_spec_i18n_str ("description",
+                                &description),
+      GNUNET_JSON_spec_mark_optional (
+        GNUNET_JSON_spec_uint64 ("quantity",
+                                 &quantity),
+        NULL),
+      GNUNET_JSON_spec_mark_optional (
+        GNUNET_JSON_spec_string ("unit",
+                                 &unit),
+        NULL),
+      GNUNET_JSON_spec_mark_optional (
+        TALER_JSON_spec_amount ("price",
+                                TMH_currency,
+                                &price),
+        NULL),
+      GNUNET_JSON_spec_mark_optional (
+        GNUNET_JSON_spec_string ("image",
+                                 &image),
+        NULL),
+      GNUNET_JSON_spec_mark_optional (
+        GNUNET_JSON_spec_json ("taxes",
+                               &taxes),
+        NULL),
+      GNUNET_JSON_spec_mark_optional (
+        GNUNET_JSON_spec_timestamp ("delivery_date",
+                                    &delivery_date),
+        NULL),
       GNUNET_JSON_spec_end ()
     };
 
@@ -90,13 +124,31 @@ check_products (const json_t *products)
     if (GNUNET_OK != res)
     {
       GNUNET_break (0);
-      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                   "Product parsing failed at #%u: %s:%u\n",
                   (unsigned int) index,
                   error_name,
                   error_line);
       return GNUNET_SYSERR;
     }
+    if ( (NULL != taxes) &&
+         (! TMH_taxes_array_valid (taxes) ) )
+    {
+      GNUNET_break (0);
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                  "Product parsing failed for taxes\n");
+      GNUNET_JSON_parse_free (spec);
+      return GNUNET_SYSERR;
+    }
+    if ( (NULL != image) &&
+         (! TMH_image_data_url_valid (image) ) )
+    {
+      GNUNET_break (0);
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  "Product parsing failed for image\n");
+      GNUNET_JSON_parse_free (spec);
+      return GNUNET_SYSERR;
+    }
     GNUNET_JSON_parse_free (spec);
   }
   return GNUNET_OK;
@@ -144,11 +196,8 @@ make_merchant_base_url (struct MHD_Connection *connection,
                                           MHD_HEADER_KIND,
                                           "X-Forwarded-Prefix");
   if (NULL != uri_path)
-  {
-    /* Currently the merchant backend is only supported at the root of the path,
-       this might change in the future.  */
-    GNUNET_assert (0);
-  }
+    GNUNET_buffer_write_path (&buf, uri_path);
+
   if (0 != strcmp (instance_id,
                    "default"))
   {
@@ -200,8 +249,8 @@ struct InventoryProduct
 static enum GNUNET_DB_QueryStatus
 execute_transaction (struct TMH_HandlerContext *hc,
                      const char *order_id,
-                     const struct GNUNET_HashCode *h_post_data,
-                     struct GNUNET_TIME_Absolute pay_deadline,
+                     const struct TALER_MerchantPostDataHashP *h_post_data,
+                     struct GNUNET_TIME_Timestamp pay_deadline,
                      const json_t *order,
                      const struct TALER_ClaimTokenP *claim_token,
                      unsigned int inventory_products_length,
@@ -211,7 +260,7 @@ execute_transaction (struct TMH_HandlerContext *hc,
                      unsigned int *out_of_stock_index)
 {
   enum GNUNET_DB_QueryStatus qs;
-  struct GNUNET_TIME_Absolute timestamp;
+  struct GNUNET_TIME_Timestamp timestamp;
   uint64_t order_serial;
 
   if (GNUNET_OK !=
@@ -228,7 +277,7 @@ execute_transaction (struct TMH_HandlerContext *hc,
                              h_post_data,
                              pay_deadline,
                              claim_token,
-                             order);
+                             order); // called 'contract terms' at database.
   if (qs <= 0)
   {
     /* qs == 0: probably instance does not exist (anymore) */
@@ -286,6 +335,10 @@ execute_transaction (struct TMH_HandlerContext *hc,
     TMH_db->rollback (TMH_db->cls);
     return qs;
   }
+  TMH_notify_order_change (hc->instance,
+                           TMH_OSF_NONE,
+                           timestamp,
+                           order_serial);
   /* finally, commit transaction (note: if it fails, we ALSO re-acquire
      the UUID locks, which is exactly what we want) */
   qs = TMH_db->commit (TMH_db->cls);
@@ -314,7 +367,7 @@ execute_transaction (struct TMH_HandlerContext *hc,
 static MHD_RESULT
 execute_order (struct MHD_Connection *connection,
                struct TMH_HandlerContext *hc,
-               const struct GNUNET_HashCode *h_post_data,
+               const struct TALER_MerchantPostDataHashP *h_post_data,
                json_t *order,
                const struct TALER_ClaimTokenP *claim_token,
                unsigned int inventory_products_length,
@@ -332,10 +385,10 @@ execute_order (struct MHD_Connection *connection,
   json_t *merchant;
   json_t *summary_i18n = NULL;
   json_t *fulfillment_i18n = NULL;
-  struct GNUNET_TIME_Absolute timestamp;
-  struct GNUNET_TIME_Absolute refund_deadline = { 0 };
-  struct GNUNET_TIME_Absolute wire_transfer_deadline;
-  struct GNUNET_TIME_Absolute pay_deadline;
+  struct GNUNET_TIME_Timestamp timestamp;
+  struct GNUNET_TIME_Timestamp refund_deadline = { 0 };
+  struct GNUNET_TIME_Timestamp wire_transfer_deadline;
+  struct GNUNET_TIME_Timestamp pay_deadline;
   struct GNUNET_JSON_Specification spec[] = {
     TALER_JSON_spec_amount ("amount",
                             TMH_currency,
@@ -353,22 +406,26 @@ execute_order (struct MHD_Connection *connection,
                            &merchant),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_json ("summary_i18n",
-                             &summary_i18n)),
+                             &summary_i18n),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_string ("fulfillment_message",
-                               &fulfillment_msg)),
+                               &fulfillment_msg),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_json ("fulfillment_message_i18n",
-                             &fulfillment_i18n)),
-    TALER_JSON_spec_absolute_time ("timestamp",
-                                   &timestamp),
+                             &fulfillment_i18n),
+      NULL),
+    GNUNET_JSON_spec_timestamp ("timestamp",
+                                &timestamp),
     GNUNET_JSON_spec_mark_optional (
-      TALER_JSON_spec_absolute_time ("refund_deadline",
-                                     &refund_deadline)),
-    TALER_JSON_spec_absolute_time ("pay_deadline",
-                                   &pay_deadline),
-    TALER_JSON_spec_absolute_time ("wire_transfer_deadline",
-                                   &wire_transfer_deadline),
+      GNUNET_JSON_spec_timestamp ("refund_deadline",
+                                  &refund_deadline),
+      NULL),
+    GNUNET_JSON_spec_timestamp ("pay_deadline",
+                                &pay_deadline),
+    GNUNET_JSON_spec_timestamp ("wire_transfer_deadline",
+                                &wire_transfer_deadline),
     GNUNET_JSON_spec_end ()
   };
   enum GNUNET_DB_QueryStatus qs;
@@ -423,7 +480,7 @@ execute_order (struct MHD_Connection *connection,
   {
     struct TALER_ClaimTokenP token;
     json_t *contract_terms;
-    struct GNUNET_HashCode orig_post;
+    struct TALER_MerchantPostDataHashP orig_post;
 
     TMH_db->preflight (TMH_db->cls);
     qs = TMH_db->lookup_order (TMH_db->cls,
@@ -562,7 +619,7 @@ execute_order (struct MHD_Connection *connection,
           "available_quantity",
           pd.total_stock - pd.total_sold - pd.total_lost),
         GNUNET_JSON_pack_allow_null (
-          GNUNET_JSON_pack_time_abs (
+          GNUNET_JSON_pack_timestamp (
             "restock_expected",
             pd.next_restock)));
       TALER_MERCHANTDB_product_details_free (&pd);
@@ -639,7 +696,7 @@ execute_order (struct MHD_Connection *connection,
 static MHD_RESULT
 patch_order (struct MHD_Connection *connection,
              struct TMH_HandlerContext *hc,
-             const struct GNUNET_HashCode *h_post_data,
+             const struct TALER_MerchantPostDataHashP *h_post_data,
              json_t *order,
              const struct TALER_ClaimTokenP *claim_token,
              struct GNUNET_TIME_Relative refund_delay,
@@ -658,13 +715,16 @@ patch_order (struct MHD_Connection *connection,
   struct TALER_Amount max_wire_fee = { 0 };
   struct TALER_Amount max_fee = { 0 };
   uint32_t wire_fee_amortization = 0;
-  struct GNUNET_TIME_Absolute timestamp = { 0 };
-  struct GNUNET_TIME_Absolute delivery_date = { 0 };
-  struct GNUNET_TIME_Absolute refund_deadline
-    = GNUNET_TIME_UNIT_FOREVER_ABS;
-  struct GNUNET_TIME_Absolute pay_deadline = { 0 };
-  struct GNUNET_TIME_Absolute wire_deadline
-    = GNUNET_TIME_UNIT_FOREVER_ABS;
+  struct GNUNET_TIME_Timestamp timestamp
+    = GNUNET_TIME_UNIT_ZERO_TS;
+  struct GNUNET_TIME_Timestamp delivery_date
+    = GNUNET_TIME_UNIT_ZERO_TS;
+  struct GNUNET_TIME_Timestamp refund_deadline
+    = GNUNET_TIME_UNIT_FOREVER_TS;
+  struct GNUNET_TIME_Timestamp pay_deadline
+    = GNUNET_TIME_UNIT_ZERO_TS;
+  struct GNUNET_TIME_Timestamp wire_deadline
+    = GNUNET_TIME_UNIT_FOREVER_TS;
   /* auto_refund only needs to be type-checked,
    * mostly because in GNUnet relative times can't
    * be negative.  */
@@ -672,49 +732,62 @@ patch_order (struct MHD_Connection *connection,
   struct GNUNET_JSON_Specification spec[] = {
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_string ("merchant_base_url",
-                               &merchant_base_url)),
+                               &merchant_base_url),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_json ("merchant",
-                             &jmerchant)),
+                             &jmerchant),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_string ("order_id",
-                               &order_id)),
+                               &order_id),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_string ("fulfillment_url",
-                               &fulfillment_url)),
+                               &fulfillment_url),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
-      TALER_JSON_spec_absolute_time ("timestamp",
-                                     &timestamp)),
+      GNUNET_JSON_spec_timestamp ("timestamp",
+                                  &timestamp),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
-      TALER_JSON_spec_absolute_time ("refund_deadline",
-                                     &refund_deadline)),
+      GNUNET_JSON_spec_timestamp ("refund_deadline",
+                                  &refund_deadline),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
-      TALER_JSON_spec_absolute_time ("pay_deadline",
-                                     &pay_deadline)),
+      GNUNET_JSON_spec_timestamp ("pay_deadline",
+                                  &pay_deadline),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
-      TALER_JSON_spec_absolute_time ("wire_transfer_deadline",
-                                     &wire_deadline)),
+      GNUNET_JSON_spec_timestamp ("wire_transfer_deadline",
+                                  &wire_deadline),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       TALER_JSON_spec_amount ("max_fee",
                               TMH_currency,
-                              &max_fee)),
+                              &max_fee),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       TALER_JSON_spec_amount ("max_wire_fee",
                               TMH_currency,
-                              &max_wire_fee)),
+                              &max_wire_fee),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_uint32 ("wire_fee_amortization",
-                               &wire_fee_amortization)),
+                               &wire_fee_amortization),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
-      TALER_JSON_spec_absolute_time ("delivery_date",
-                                     &delivery_date)),
+      GNUNET_JSON_spec_timestamp ("delivery_date",
+                                  &delivery_date),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
-      TALER_JSON_spec_relative_time ("auto_refund",
-                                     &auto_refund)),
+      GNUNET_JSON_spec_relative_time ("auto_refund",
+                                      &auto_refund),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_json ("delivery_location",
-                             &delivery_location)),
-
+                             &delivery_location),
+      NULL),
     GNUNET_JSON_spec_end ()
   };
   enum GNUNET_GenericReturnValue ret;
@@ -745,6 +818,7 @@ patch_order (struct MHD_Connection *connection,
     tm_info = localtime (&timer);
     if (NULL == tm_info)
     {
+      GNUNET_JSON_parse_free (spec);
       return TALER_MHD_reply_with_error (
         connection,
         MHD_HTTP_INTERNAL_SERVER_ERROR,
@@ -795,8 +869,8 @@ patch_order (struct MHD_Connection *connection,
       if (strstr (pos + strlen ("${ORDER_ID}"),
                   "${ORDER_ID}"))
       {
-        /* FIXME: free anything? */
         GNUNET_break_op (0);
+        GNUNET_JSON_parse_free (spec);
         return TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_BAD_REQUEST,
                                            TALER_EC_GENERIC_PARAMETER_MALFORMED,
@@ -824,20 +898,19 @@ patch_order (struct MHD_Connection *connection,
   /* Check soundness of refund deadline, and that a timestamp
    * is actually present.  */
   {
-    struct GNUNET_TIME_Absolute now = GNUNET_TIME_absolute_get ();
+    struct GNUNET_TIME_Timestamp now = GNUNET_TIME_timestamp_get ();
 
-    (void) GNUNET_TIME_round_abs (&now);
     /* Add timestamp if it doesn't exist (or is zero) */
-    if (0 == timestamp.abs_value_us)
+    if (GNUNET_TIME_absolute_is_zero (timestamp.abs_time))
     {
       GNUNET_assert (0 ==
                      json_object_set_new (order,
                                           "timestamp",
-                                          GNUNET_JSON_from_time_abs (now)));
+                                          GNUNET_JSON_from_timestamp (now)));
     }
 
     /* If no refund_deadline given, set one based on refund_delay.  */
-    if (GNUNET_TIME_absolute_is_never (refund_deadline))
+    if (GNUNET_TIME_absolute_is_never (refund_deadline.abs_time))
     {
       if (GNUNET_TIME_relative_is_zero (refund_delay))
       {
@@ -847,20 +920,22 @@ patch_order (struct MHD_Connection *connection,
       }
       else
       {
-        refund_deadline = GNUNET_TIME_relative_to_absolute (refund_delay);
-        (void) GNUNET_TIME_round_abs (&refund_deadline);
+        refund_deadline = GNUNET_TIME_relative_to_timestamp (refund_delay);
       }
 
       GNUNET_assert (0 ==
                      json_object_set_new (order,
                                           "refund_deadline",
-                                          GNUNET_JSON_from_time_abs (
+                                          GNUNET_JSON_from_timestamp (
                                             refund_deadline)));
     }
-    if ((0 != delivery_date.abs_value_us) &&
-        (delivery_date.abs_value_us < now.abs_value_us) )
+    if ( (! GNUNET_TIME_absolute_is_zero (delivery_date.abs_time)) &&
+         (GNUNET_TIME_timestamp_cmp (delivery_date,
+                                     <,
+                                     now)) )
     {
       GNUNET_break_op (0);
+      GNUNET_JSON_parse_free (spec);
       return TALER_MHD_reply_with_error (
         connection,
         MHD_HTTP_BAD_REQUEST,
@@ -869,43 +944,54 @@ patch_order (struct MHD_Connection *connection,
     }
   }
 
-  if (0 == pay_deadline.abs_value_us)
+  if (GNUNET_TIME_absolute_is_zero (pay_deadline.abs_time))
   {
-    struct GNUNET_TIME_Absolute t;
+    struct GNUNET_TIME_Timestamp t;
 
-    t = GNUNET_TIME_relative_to_absolute (settings->default_pay_delay);
-    (void) GNUNET_TIME_round_abs (&t);
+    t = GNUNET_TIME_relative_to_timestamp (settings->default_pay_delay);
     GNUNET_assert (0 ==
                    json_object_set_new (order,
                                         "pay_deadline",
-                                        GNUNET_JSON_from_time_abs (t)));
+                                        GNUNET_JSON_from_timestamp (t)));
   }
 
-  if (GNUNET_TIME_UNIT_FOREVER_ABS.abs_value_us == wire_deadline.abs_value_us)
+  if (GNUNET_TIME_absolute_is_never (wire_deadline.abs_time))
   {
-    struct GNUNET_TIME_Absolute t;
+    struct GNUNET_TIME_Timestamp t;
 
-    t = GNUNET_TIME_relative_to_absolute (
+    t = GNUNET_TIME_relative_to_timestamp (
       GNUNET_TIME_relative_max (settings->default_wire_transfer_delay,
                                 refund_delay));
-    wire_deadline = GNUNET_TIME_absolute_max (refund_deadline,
-                                              t);
-    (void) GNUNET_TIME_round_abs (&wire_deadline);
+    wire_deadline = GNUNET_TIME_timestamp_max (refund_deadline,
+                                               t);
+    if (GNUNET_TIME_absolute_is_never (wire_deadline.abs_time))
+    {
+      GNUNET_break_op (0);
+      GNUNET_JSON_parse_free (spec);
+      return TALER_MHD_reply_with_error (
+        connection,
+        MHD_HTTP_BAD_REQUEST,
+        TALER_EC_MERCHANT_PRIVATE_POST_ORDERS_WIRE_DEADLINE_IS_NEVER,
+        "order:wire_transfer_deadline");
+
+    }
     GNUNET_assert (0 ==
                    json_object_set_new (order,
                                         "wire_transfer_deadline",
-                                        GNUNET_JSON_from_time_abs (
+                                        GNUNET_JSON_from_timestamp (
                                           wire_deadline)));
   }
-  if (wire_deadline.abs_value_us < refund_deadline.abs_value_us)
+  if (GNUNET_TIME_timestamp_cmp (wire_deadline,
+                                 <,
+                                 refund_deadline))
   {
     GNUNET_break_op (0);
+    GNUNET_JSON_parse_free (spec);
     return TALER_MHD_reply_with_error (
       connection,
       MHD_HTTP_BAD_REQUEST,
       TALER_EC_MERCHANT_PRIVATE_POST_ORDERS_REFUND_AFTER_WIRE_DEADLINE,
       "order:wire_transfer_deadline;order:refund_deadline");
-
   }
 
   /* Note: total amount currency match checked
@@ -927,8 +1013,8 @@ patch_order (struct MHD_Connection *connection,
                    json_object_set_new (
                      order,
                      "max_fee",
-                     TALER_JSON_from_amount
-                       (&settings->default_max_deposit_fee)));
+                     TALER_JSON_from_amount (
+                       &settings->default_max_deposit_fee)));
   }
   if (0 == wire_fee_amortization)
   {
@@ -955,6 +1041,7 @@ patch_order (struct MHD_Connection *connection,
            ('/' != merchant_base_url[strlen (merchant_base_url) - 1]))
   {
     GNUNET_break_op (0);
+    GNUNET_JSON_parse_free (spec);
     return TALER_MHD_reply_with_error (
       connection,
       MHD_HTTP_BAD_REQUEST,
@@ -966,48 +1053,63 @@ patch_order (struct MHD_Connection *connection,
   if (NULL != jmerchant)
   {
     GNUNET_break_op (0);
+    GNUNET_JSON_parse_free (spec);
     return TALER_MHD_reply_with_error (
       connection,
       MHD_HTTP_BAD_REQUEST,
       TALER_EC_MERCHANT_PRIVATE_POST_ORDERS_PROPOSAL_PARSE_ERROR,
       "'merchant' field already set, but must be provided by backend");
   }
-  jmerchant = GNUNET_JSON_PACK (
-    GNUNET_JSON_pack_string ("name",
-                             settings->name));
-  GNUNET_assert (NULL != jmerchant);
-  {
-    json_t *loca;
 
-    /* Handle merchant address */
-    loca = settings->address;
-    if (NULL != loca)
-    {
-      loca = json_deep_copy (loca);
-      GNUNET_assert (0 ==
-                     json_object_set_new (jmerchant,
-                                          "address",
-                                          loca));
-    }
-  }
   {
-    json_t *locj;
+    json_t *jm;
 
-    /* Handle merchant jurisdiction */
-    locj = settings->jurisdiction;
-    if (NULL != locj)
+    jm = GNUNET_JSON_PACK (
+      GNUNET_JSON_pack_string ("name",
+                               settings->name),
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_string ("website",
+                                 settings->website)),
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_string ("email",
+                                 settings->email)),
+      GNUNET_JSON_pack_allow_null (
+        GNUNET_JSON_pack_string ("logo",
+                                 settings->logo)));
+    GNUNET_assert (NULL != jm);
     {
-      locj = json_deep_copy (locj);
-      GNUNET_assert (0 ==
-                     json_object_set_new (jmerchant,
-                                          "jurisdiction",
-                                          locj));
+      json_t *loca;
+
+      /* Handle merchant address */
+      loca = settings->address;
+      if (NULL != loca)
+      {
+        loca = json_deep_copy (loca);
+        GNUNET_assert (0 ==
+                       json_object_set_new (jm,
+                                            "address",
+                                            loca));
+      }
     }
+    {
+      json_t *locj;
+
+      /* Handle merchant jurisdiction */
+      locj = settings->jurisdiction;
+      if (NULL != locj)
+      {
+        locj = json_deep_copy (locj);
+        GNUNET_assert (0 ==
+                       json_object_set_new (jm,
+                                            "jurisdiction",
+                                            locj));
+      }
+    }
+    GNUNET_assert (0 ==
+                   json_object_set_new (order,
+                                        "merchant",
+                                        jm));
   }
-  GNUNET_assert (0 ==
-                 json_object_set_new (order,
-                                      "merchant",
-                                      jmerchant));
 
   /* add fields to the contract that the backend should provide */
   GNUNET_assert (0 ==
@@ -1026,6 +1128,7 @@ patch_order (struct MHD_Connection *connection,
   if (GNUNET_OK !=
       TALER_JSON_contract_seed_forgettable (order))
   {
+    GNUNET_JSON_parse_free (spec);
     return TALER_MHD_reply_with_error (
       connection,
       MHD_HTTP_BAD_REQUEST,
@@ -1046,19 +1149,21 @@ patch_order (struct MHD_Connection *connection,
 
   /* sanity check result */
   {
-    struct GNUNET_HashCode h_control;
+    struct TALER_PrivateContractHashP h_control;
 
     switch (TALER_JSON_contract_hash (order,
                                       &h_control))
     {
     case GNUNET_SYSERR:
       GNUNET_break (0);
+      GNUNET_JSON_parse_free (spec);
       return TALER_MHD_reply_with_error (
         connection,
         MHD_HTTP_INTERNAL_SERVER_ERROR,
         TALER_EC_GENERIC_FAILED_COMPUTE_JSON_HASH,
         "could not compute hash of patched order");
     case GNUNET_NO:
+      GNUNET_JSON_parse_free (spec);
       GNUNET_break_op (0);
       return TALER_MHD_reply_with_error (
         connection,
@@ -1069,15 +1174,21 @@ patch_order (struct MHD_Connection *connection,
       break;
     }
   }
-  return execute_order (connection,
-                        hc,
-                        h_post_data,
-                        order,
-                        claim_token,
-                        inventory_products_length,
-                        inventory_products,
-                        uuids_length,
-                        uuids);
+  {
+    MHD_RESULT mres;
+
+    mres = execute_order (connection,
+                          hc,
+                          h_post_data,
+                          order,
+                          claim_token,
+                          inventory_products_length,
+                          inventory_products,
+                          uuids_length,
+                          uuids);
+    GNUNET_JSON_parse_free (spec);
+    return mres;
+  }
 }
 
 
@@ -1102,7 +1213,7 @@ patch_order (struct MHD_Connection *connection,
 static MHD_RESULT
 add_payment_details (struct MHD_Connection *connection,
                      struct TMH_HandlerContext *hc,
-                     const struct GNUNET_HashCode *h_post_data,
+                     const struct TALER_MerchantPostDataHashP *h_post_data,
                      json_t *order,
                      const struct TALER_ClaimTokenP *claim_token,
                      struct GNUNET_TIME_Relative refund_delay,
@@ -1175,7 +1286,7 @@ add_payment_details (struct MHD_Connection *connection,
 static MHD_RESULT
 merge_inventory (struct MHD_Connection *connection,
                  struct TMH_HandlerContext *hc,
-                 const struct GNUNET_HashCode *h_post_data,
+                 const struct TALER_MerchantPostDataHashP *h_post_data,
                  json_t *order,
                  const struct TALER_ClaimTokenP *claim_token,
                  struct GNUNET_TIME_Relative refund_delay,
@@ -1240,6 +1351,9 @@ merge_inventory (struct MHD_Connection *connection,
           ec = TALER_EC_GENERIC_DB_SOFT_FAILURE;
           break;
         case GNUNET_DB_STATUS_SUCCESS_NO_RESULTS:
+          GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                      "Product %s from order unknown\n",
+                      inventory_products[i].product_id);
           http_status = MHD_HTTP_NOT_FOUND;
           ec = TALER_EC_MERCHANT_GENERIC_PRODUCT_UNKNOWN;
           break;
@@ -1337,24 +1451,29 @@ TMH_private_post_orders (const struct TMH_RequestHandler *rh,
     GNUNET_JSON_spec_json ("order",
                            &order),
     GNUNET_JSON_spec_mark_optional (
-      TALER_JSON_spec_relative_time ("refund_delay",
-                                     &refund_delay)),
+      GNUNET_JSON_spec_relative_time ("refund_delay",
+                                      &refund_delay),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_string ("payment_target",
-                               &payment_target)),
+                               &payment_target),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_json ("inventory_products",
-                             &ip)),
+                             &ip),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_json ("lock_uuids",
-                             &uuid)),
+                             &uuid),
+      NULL),
     GNUNET_JSON_spec_mark_optional (
       GNUNET_JSON_spec_bool ("create_token",
-                             &create_token)),
+                             &create_token),
+      NULL),
     GNUNET_JSON_spec_end ()
   };
   enum GNUNET_GenericReturnValue ret;
-  struct GNUNET_HashCode h_post_data;
+  struct TALER_MerchantPostDataHashP h_post_data;
 
   (void) rh;
   ret = TALER_MHD_parse_json_data (connection,
@@ -1367,8 +1486,8 @@ TMH_private_post_orders (const struct TMH_RequestHandler *rh,
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Refund delay is %s\n",
-              GNUNET_STRINGS_relative_time_to_string (refund_delay,
-                                                      GNUNET_NO));
+              GNUNET_TIME_relative2s (refund_delay,
+                                      false));
 
   TMH_db->expire_locks (TMH_db->cls);
   if (create_token)
@@ -1404,7 +1523,7 @@ TMH_private_post_orders (const struct TMH_RequestHandler *rh,
     }
     GNUNET_CRYPTO_hash (req_body_enc,
                         strlen (req_body_enc),
-                        &h_post_data);
+                        &h_post_data.hash);
     GNUNET_free (req_body_enc);
   }
 
@@ -1478,21 +1597,10 @@ TMH_private_post_orders (const struct TMH_RequestHandler *rh,
                        json_array_size (uuid));
     for (unsigned int i = 0; i<uuids_len; i++)
     {
-      const char *error_name;
-      unsigned int error_line;
-      const char *uuidsi;
-      struct GNUNET_JSON_Specification ispec[] = {
-        GNUNET_JSON_spec_string ("uuid",
-                                 &uuidsi),
-        GNUNET_JSON_spec_end ()
-      };
+      json_t *ui = json_array_get (uuid,
+                                   i);
 
-      ret = GNUNET_JSON_parse (json_array_get (uuid,
-                                               i),
-                               ispec,
-                               &error_name,
-                               &error_line);
-      if (GNUNET_OK != ret)
+      if (! json_is_string (ui))
       {
         GNUNET_break_op (0);
         GNUNET_array_grow (ips,
@@ -1503,16 +1611,14 @@ TMH_private_post_orders (const struct TMH_RequestHandler *rh,
                            0);
         GNUNET_JSON_parse_free (spec);
         GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                    "UUID parsing failed at #%u: %s:%u\n",
-                    i,
-                    error_name,
-                    error_line);
+                    "UUID parsing failed at #%u\n",
+                    i);
         return TALER_MHD_reply_with_error (connection,
                                            MHD_HTTP_BAD_REQUEST,
                                            TALER_EC_GENERIC_PARAMETER_MALFORMED,
                                            "lock_uuids");
       }
-      TMH_uuid_from_string (uuidsi,
+      TMH_uuid_from_string (json_string_value (ui),
                             &uuids[i]);
     }
   }
