@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2014-2021 Taler Systems SA
+  (C) 2014-2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Affero General Public License as published by the Free Software
@@ -135,7 +135,16 @@ struct PostTransfersContext
    * #GNUNET_SYSERR if we found a matching coin, but the amounts do not match.
    * #GNUNET_OK if we did find a matching coin.
    */
-  int check_transfer_result;
+  enum GNUNET_GenericReturnValue check_transfer_result;
+
+  /**
+   * Did we suspend @a connection and are thus in
+   * the #ptc_head DLL (#GNUNET_YES). Set to
+   * #GNUNET_NO if we are not suspended, and to
+   * #GNUNET_SYSERR if we should close the connection
+   * without a response due to shutdown.
+   */
+  enum GNUNET_GenericReturnValue suspended;
 
   /**
    * Should we retry the transaction due to a serialization error?
@@ -147,10 +156,6 @@ struct PostTransfersContext
    */
   bool downloaded;
 
-  /**
-   * Are we currently suspended?
-   */
-  bool suspended;
 };
 
 
@@ -175,7 +180,7 @@ TMH_force_post_transfers_resume ()
     GNUNET_CONTAINER_DLL_remove (ptc_head,
                                  ptc_tail,
                                  ptc);
-    ptc->suspended = false;
+    ptc->suspended = GNUNET_SYSERR;
     MHD_resume_connection (ptc->connection);
     if (NULL != ptc->timeout_task)
     {
@@ -213,7 +218,7 @@ resume_transfer_with_response (struct PostTransfersContext *ptc,
   GNUNET_CONTAINER_DLL_remove (ptc_head,
                                ptc_tail,
                                ptc);
-  ptc->suspended = false;
+  ptc->suspended = GNUNET_NO;
   MHD_resume_connection (ptc->connection);
   TALER_MHD_daemon_trigger ();   /* we resumed, kick MHD */
 }
@@ -298,9 +303,9 @@ check_transfer (void *cls,
                 const struct TALER_Amount *deposit_fee,
                 const struct TALER_Amount *refund_fee,
                 const struct TALER_Amount *wire_fee,
-                const struct GNUNET_HashCode *h_wire,
-                struct GNUNET_TIME_Absolute deposit_timestamp,
-                struct GNUNET_TIME_Absolute refund_deadline,
+                const struct TALER_MerchantWireHashP *h_wire,
+                struct GNUNET_TIME_Timestamp deposit_timestamp,
+                struct GNUNET_TIME_Timestamp refund_deadline,
                 const struct TALER_ExchangeSignatureP *exchange_sig,
                 const struct TALER_ExchangePublicKeyP *exchange_pub)
 {
@@ -326,10 +331,10 @@ check_transfer (void *cls,
             TALER_EC_MERCHANT_PRIVATE_POST_TRANSFERS_CONFLICTING_REPORTS),
           GNUNET_JSON_pack_string ("exchange_url",
                                    exchange_url),
-          GNUNET_JSON_pack_time_abs ("deposit_timestamp",
-                                     deposit_timestamp),
-          GNUNET_JSON_pack_time_abs ("refund_deadline",
-                                     refund_deadline),
+          GNUNET_JSON_pack_timestamp ("deposit_timestamp",
+                                      deposit_timestamp),
+          GNUNET_JSON_pack_timestamp ("refund_deadline",
+                                      refund_deadline),
           GNUNET_JSON_pack_uint64 ("conflict_offset",
                                    ptc->current_offset),
           GNUNET_JSON_pack_data_auto ("coin_pub",
@@ -370,16 +375,15 @@ check_transfer (void *cls,
  * @return #GNUNET_SYSERR if we returned hard proof of
  *   missbehavior from the exchange to the client
  */
-static int
+static enum GNUNET_GenericReturnValue
 check_wire_fee (struct PostTransfersContext *ptc,
-                struct GNUNET_TIME_Absolute execution_time,
+                struct GNUNET_TIME_Timestamp execution_time,
                 const struct TALER_Amount *wire_fee)
 {
-  struct TALER_Amount expected_fee;
-  struct TALER_Amount closing_fee;
+  struct TALER_WireFeeSet fees;
   struct TALER_MasterSignatureP master_sig;
-  struct GNUNET_TIME_Absolute start_date;
-  struct GNUNET_TIME_Absolute end_date;
+  struct GNUNET_TIME_Timestamp start_date;
+  struct GNUNET_TIME_Timestamp end_date;
   enum GNUNET_DB_QueryStatus qs;
   char *wire_method;
 
@@ -388,8 +392,7 @@ check_wire_fee (struct PostTransfersContext *ptc,
                                 &ptc->master_pub,
                                 wire_method,
                                 execution_time,
-                                &expected_fee,
-                                &closing_fee,
+                                &fees,
                                 &start_date,
                                 &end_date,
                                 &master_sig);
@@ -409,14 +412,14 @@ check_wire_fee (struct PostTransfersContext *ptc,
                 "Failed to find wire fee for `%s' and method `%s' at %s in DB, accepting blindly that the fee is %s\n",
                 TALER_B2S (&ptc->master_pub),
                 wire_method,
-                GNUNET_STRINGS_absolute_time_to_string (execution_time),
+                GNUNET_TIME_timestamp2s (execution_time),
                 TALER_amount2s (wire_fee));
     GNUNET_free (wire_method);
     return GNUNET_NO;
   case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
     break;
   }
-  if (0 <= TALER_amount_cmp (&expected_fee,
+  if (0 <= TALER_amount_cmp (&fees.wire,
                              wire_fee))
   {
     GNUNET_free (wire_method);
@@ -430,16 +433,16 @@ check_wire_fee (struct PostTransfersContext *ptc,
         TALER_EC_MERCHANT_PRIVATE_POST_TRANSFERS_BAD_WIRE_FEE),
       TALER_JSON_pack_amount ("wire_fee",
                               wire_fee),
-      GNUNET_JSON_pack_time_abs ("execution_time",
-                                 execution_time),
+      GNUNET_JSON_pack_timestamp ("execution_time",
+                                  execution_time),
       TALER_JSON_pack_amount ("expected_wire_fee",
-                              &expected_fee),
+                              &fees.wire),
       TALER_JSON_pack_amount ("expected_closing_fee",
-                              &closing_fee),
-      GNUNET_JSON_pack_time_abs ("start_date",
-                                 start_date),
-      GNUNET_JSON_pack_time_abs ("end_date",
-                                 end_date),
+                              &fees.closing),
+      GNUNET_JSON_pack_timestamp ("start_date",
+                                  start_date),
+      GNUNET_JSON_pack_timestamp ("end_date",
+                                  end_date),
       GNUNET_JSON_pack_data_auto ("master_sig",
                                   &master_sig),
       GNUNET_JSON_pack_data_auto ("master_pub",
@@ -891,7 +894,7 @@ download (struct PostTransfersContext *ptc)
   GNUNET_log (GNUNET_ERROR_TYPE_INFO,
               "Suspending POST /private/transfers handling while working with exchange\n");
   MHD_suspend_connection (ptc->connection);
-  ptc->suspended = true;
+  ptc->suspended = GNUNET_YES;
   GNUNET_CONTAINER_DLL_insert (ptc_head,
                                ptc_tail,
                                ptc);
@@ -923,6 +926,8 @@ TMH_private_post_transfers (const struct TMH_RequestHandler *rh,
     hc->ctx = ptc;
     hc->cc = &transfer_cleanup;
   }
+  if (GNUNET_SYSERR == ptc->suspended)
+    return MHD_NO; /* we are in shutdown */
   /* resume logic: did we get resumed after a reply was built? */
   if (0 != ptc->response_code)
     return queue (ptc);
@@ -930,7 +935,7 @@ TMH_private_post_transfers (const struct TMH_RequestHandler *rh,
        (NULL != ptc->wdh) )
   {
     /* likely old MHD version causing spurious wake-up */
-    GNUNET_break (ptc->suspended);
+    GNUNET_break (GNUNET_NO == ptc->suspended);
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Not sure why we are here, should be suspended\n");
     return MHD_YES; /* still work in progress */
@@ -969,7 +974,7 @@ TMH_private_post_transfers (const struct TMH_RequestHandler *rh,
   /* Check if transfer data is in database, if not, add it. */
   for (unsigned int retry = 0; retry<MAX_RETRIES; retry++)
   {
-    struct GNUNET_TIME_Absolute execution_time;
+    struct GNUNET_TIME_Timestamp execution_time;
     struct TALER_Amount total_amount;
     struct TALER_Amount exchange_amount;
     struct TALER_Amount wire_fee;
@@ -1047,6 +1052,10 @@ TMH_private_post_transfers (const struct TMH_RequestHandler *rh,
                                              TALER_EC_MERCHANT_PRIVATE_POST_TRANSFERS_ACCOUNT_NOT_FOUND,
                                              ptc->payto_uri);
         case GNUNET_DB_STATUS_SUCCESS_ONE_RESULT:
+          GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                      "Bank account `%s' is configured at row %llu\n",
+                      ptc->payto_uri,
+                      (unsigned long long) account_serial);
           break;
         }
 
@@ -1328,8 +1337,8 @@ TMH_private_post_transfers (const struct TMH_RequestHandler *rh,
                                   &total_amount),
           TALER_JSON_pack_amount ("wire_fee",
                                   &wire_fee),
-          GNUNET_JSON_pack_time_abs ("execution_time",
-                                     execution_time),
+          GNUNET_JSON_pack_timestamp ("execution_time",
+                                      execution_time),
           GNUNET_JSON_pack_array_steal ("deposit_sums",
                                         deposit_sums));
       } /* end of 'verified == true' (not an 'if'!) */
