@@ -17,6 +17,7 @@
  * @file merchantdb_helper.c
  * @brief Helper functions for the merchant database logic
  * @author Christian Grothoff
+ * @author Priscilla Huang
  */
 #include "platform.h"
 #include <taler/taler_util.h>
@@ -36,4 +37,38 @@ TALER_MERCHANTDB_product_details_free (
 }
 
 
+
+void
+TALER_MERCHANTDB_template_details_free (
+  struct TALER_MERCHANTDB_TemplateDetails *tp)
+{
+  GNUNET_free (tp->template_description);
+  GNUNET_free (tp->image);
+  json_decref (tp->template_contract);
+}
+
+
+void
+TALER_MERCHANTDB_webhook_details_free (
+  struct TALER_MERCHANTDB_WebhookDetails *wb)
+{
+  GNUNET_free (wb->event_type);
+  GNUNET_free (wb->url);
+  GNUNET_free (wb->http_method);
+  GNUNET_free (wb->header_template);
+  GNUNET_free (wb->body_template);
+}
+
+void
+TALER_MERCHANTDB_pending_webhook_details_free (
+  struct TALER_MERCHANTDB_PendingWebhookDetails *pwb)
+{
+  GNUNET_free (pwb->url);
+  GNUNET_free (pwb->http_method);
+  GNUNET_free (pwb->header);
+  GNUNET_free (pwb->body);
+}
+
+
+
 /* end of merchantdb_helper.c */
