@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  (C) 2018-2021 Taler Systems SA
+  (C) 2018-2022 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as
@@ -21,7 +21,8 @@
  * @file taler_merchant_testing_lib.h
  * @brief API for writing an interpreter to test Taler components
  * @author Christian Grothoff <christian@grothoff.org>
- * @author Marcello Stanisci
+ * @author Marcello Shtanisci
+ * @author Priscilla HUANG
  */
 #ifndef TALER_MERCHANT_TESTING_LIB_H
 #define TALER_MERCHANT_TESTING_LIB_H
@@ -1487,6 +1488,286 @@ TALER_TESTING_cmd_merchant_kyc_get (const char *label,
                                     unsigned int expected_http_status);
 
 
+/* ****** Templates ******* */
+
+/**
+ * Define a "POST /templates" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        POST /templates request.
+ * @param template_id the ID of the template to query
+ * @param template_description description of the template
+ * @param image base64-encoded template image
+ * @param template_contract where the contract of the company is
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_templates2 (
+  const char *label,
+  const char *merchant_url,
+  const char *template_id,
+  const char *template_description,
+  const char *image,
+  json_t *template_contract,
+  unsigned int http_status);
+
+
+/**
+ * Define a "POST /templates" CMD, simple version
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        POST /templates request.
+ * @param template_id the ID of the template to create
+ * @param template_description description of the template
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_templates (const char *label,
+                                           const char *merchant_url,
+                                           const char *template_id,
+                                           const char *template_description,
+                                           unsigned int http_status);
+
+
+/**
+ * Define a "PATCH /templates/$ID" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        PATCH /template request.
+ * @param template_id the ID of the template to query
+ * @param template_description description of the template
+ * @param image base64-encoded template image
+ * @param template_contract contract of the company
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_patch_template (
+  const char *label,
+  const char *merchant_url,
+  const char *template_id,
+  const char *template_description,
+  const char *image,
+  json_t *template_contract,
+  unsigned int http_status);
+
+
+/**
+ * Define a "GET /templates" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        GET /templates request.
+ * @param http_status expected HTTP response code.
+ * @param ... NULL-terminated list of labels (const char *) of
+ *        template (commands) we expect to be returned in the list
+ *        (assuming @a http_code is #MHD_HTTP_OK)
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_get_templates (const char *label,
+                                          const char *merchant_url,
+                                          unsigned int http_status,
+                                          ...);
+
+
+/**
+ * Define a "GET template" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        GET /templates/$ID request.
+ * @param template_id the ID of the template to query
+ * @param http_status expected HTTP response code.
+ * @param template_reference reference to a "POST /templates" or "PATCH /templates/$ID" CMD
+ *        that will provide what we expect the backend to return to us
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_get_template (const char *label,
+                                         const char *merchant_url,
+                                         const char *template_id,
+                                         unsigned int http_status,
+                                         const char *template_reference);
+
+
+/**
+ * Define a "DELETE template" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        DELETE /templates/$ID request.
+ * @param template_id the ID of the template to query
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_delete_template (const char *label,
+                                            const char *merchant_url,
+                                            const char *template_id,
+                                            unsigned int http_status);
+
+
+/**
+ * Define a "POST /using-templates" CMD, simple version
+ *
+ * @param label command label.
+ * @param template_ref label of command that created the template to use
+ * @param merchant_url base URL of the merchant serving the
+ *        POST /using-templates request.
+ * @param summary given by the customer to know what they did pay
+ * @param amount given by the customer to pay
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_using_templates (const char *label,
+                                                 const char *template_ref,
+                                                 const char *merchant_url,
+                                                 const char *summary,
+                                                 const char *amount,
+                                                 unsigned int http_status);
+
+
+/* ****** Webhooks ******* */
+
+
+/**
+ * Define a "POST /webhooks" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        POST /webhooks request.
+ * @param webhook_id the ID of the webhook to query
+ * @param event_type event of the webhook
+ * @param url use by the customer
+ * @param http_method use by the merchant
+ * @param header_template of the webhook
+ * @param body_template of the webhook
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_webhooks2 (
+  const char *label,
+  const char *merchant_url,
+  const char *webhook_id,
+  const char *event_type,
+  const char *url,
+  const char *http_method,
+  const char *header_template,
+  const char *body_template,
+  unsigned int http_status);
+
+
+/**
+ * Define a "POST /webhooks" CMD, simple version
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        POST /webhooks request.
+ * @param webhook_id the ID of the webhook to query
+ * @param event_type event of the webhook
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_post_webhooks (const char *label,
+                                          const char *merchant_url,
+                                          const char *webhook_id,
+                                          const char *event_type,
+                                          unsigned int http_status);
+
+
+/**
+ * Define a "PATCH /webhooks/$ID" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        PATCH /webhook request.
+ * @param webhook_id the ID of the webhook to query
+ * @param event_type event of the webhook
+ * @param url use by the customer
+ * @param http_method use by the merchant
+ * @param header_template of the webhook
+ * @param body_template of the webhook
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_patch_webhook (
+  const char *label,
+  const char *merchant_url,
+  const char *webhook_id,
+  const char *event_type,
+  const char *url,
+  const char *http_method,
+  const char *header_template,
+  const char *body_template,
+  unsigned int http_status);
+
+
+/**
+ * Define a "GET /webhooks" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        GET /webhooks request.
+ * @param http_status expected HTTP response code.
+ * @param ... NULL-terminated list of labels (const char *) of
+ *        webhook (commands) we expect to be returned in the list
+ *        (assuming @a http_code is #MHD_HTTP_OK)
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_get_webhooks (const char *label,
+                                         const char *merchant_url,
+                                         unsigned int http_status,
+                                         ...);
+
+
+/**
+ * Define a "GET webhook" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        GET /templates/$ID request.
+ * @param webhook_id the ID of the webhook to query
+ * @param http_status expected HTTP response code.
+ * @param webhook_reference reference to a "POST /webhooks" or "PATCH /webhooks/$ID" CMD
+ *        that will provide what we expect the backend to return to us
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_get_webhook (const char *label,
+                                        const char *merchant_url,
+                                        const char *webhook_id,
+                                        unsigned int http_status,
+                                        const char *webhook_reference);
+
+
+/**
+ * Define a "DELETE webhook" CMD.
+ *
+ * @param label command label.
+ * @param merchant_url base URL of the merchant serving the
+ *        DELETE /webhooks/$ID request.
+ * @param webhook_id the ID of the webhook to query
+ * @param http_status expected HTTP response code.
+ * @return the command.
+ */
+struct TALER_TESTING_Command
+TALER_TESTING_cmd_merchant_delete_webhook (const char *label,
+                                           const char *merchant_url,
+                                           const char *webhook_id,
+                                           unsigned int http_status);
+
+
 /* ****** Specific traits supported by this component ******* */
 
 
@@ -1527,7 +1808,18 @@ TALER_TESTING_cmd_merchant_kyc_get (const char *label,
   op (order_terms, const json_t) \
   op (h_contract_terms, const struct TALER_PrivateContractHashP) \
   op (h_wire, const struct TALER_MerchantWireHashP) \
-  op (proposal_reference, const char *)
+  op (proposal_reference, const char *) \
+  op (template_description, const char *) \
+  op (template_image, const char *) \
+  op (template_id, const char *) \
+  op (template_contract, const json_t) \
+  op (event_type, const char *)   \
+  op (url, const char *) \
+  op (webhook_id, const char *) \
+  op (http_method, const char *)   \
+  op (header_template, const char *)   \
+  op (body_template, const char *)   \
+  op (summary, const char *)
 
 
 /**
