@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2021 Taler Systems SA
+  Copyright (C) 2014-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify it under the
   terms of the GNU Lesser General Public License as published by the Free Software
@@ -18,6 +18,7 @@
  * @brief database access for the merchant
  * @author Florian Dold
  * @author Christian Grothoff
+ * @author Priscilla Huang
  */
 #ifndef TALER_MERCHANTDB_PLUGIN_H
 #define TALER_MERCHANTDB_PLUGIN_H
@@ -216,7 +217,6 @@ typedef void
   unsigned int accounts_length,
   const struct TALER_MERCHANTDB_AccountDetails accounts[]);
 
-
 /**
  * Typically called by `lookup_products`.
  *
@@ -302,6 +302,174 @@ struct TALER_MERCHANTDB_ProductDetails
 };
 
 
+/**
+ * Typically called by `lookup_templates`.
+ *
+ * @param cls a `json_t *` JSON array to build
+ * @param template_id ID of the template
+ * @param template_description description of the template
+ */
+typedef void
+(*TALER_MERCHANTDB_TemplatesCallback)(void *cls,
+                                      const char *template_id,
+                                      const char *template_description);
+
+
+/**
+ * Details about a template.
+ */
+struct TALER_MERCHANTDB_TemplateDetails
+{
+  /**
+   * Description of the template.
+   */
+  char *template_description;
+
+  /**
+   * Base64-encoded image, or NULL.
+   */
+  char *image;
+
+  /**
+   * In this template contract, we can have additional information.
+   */
+  json_t *template_contract;
+};
+
+
+/**
+ * Typically called by `lookup_webhooks`.
+ *
+ * @param cls a `json_t *` JSON array to build
+ * @param webhook_id ID of the webhook
+ * @param event_type event of the webhook
+ */
+typedef void
+(*TALER_MERCHANTDB_WebhooksCallback)(void *cls,
+                                     const char *webhook_id,
+                                     const char *event_type);
+
+
+/**
+ * Details about a webhook.
+ */
+struct TALER_MERCHANTDB_WebhookDetails
+{
+
+  /**
+   * event of the webhook.
+   */
+  char *event_type;
+
+  /**
+   * URL of the webhook. The customer will be redirected on this url.
+   */
+  char *url;
+
+  /**
+   * Http method used by the webhook.
+   */
+  char *http_method;
+
+  /**
+   * Header template of the webhook.
+   */
+  char *header_template;
+
+  /**
+   * Body template of the webhook.
+   */
+  char *body_template;
+
+};
+
+
+/**
+ * Typically called by `lookup_webhook_by_event`.
+ *
+ * @param cls a `json_t *` JSON array to build
+ * @param webhook_serial reference to the configured webhook template.
+ * @param event_type which type of event triggers this type of webhook
+ * @param url the HTTP URL to make the webhook request to
+ * @param http_method HTTP method use for the webhook
+ * @param header_template template for the header of the webhook
+ * @param body_template template for the body of the webhook
+ */
+typedef void
+(*TALER_MERCHANTDB_WebhookDetailCallback)(void *cls,
+                                          uint64_t webhook_serial,
+                                          const char *event_type,
+                                          const char *url,
+                                          const char *http_method,
+                                          const char *header_template,
+                                          const char *body_template);
+
+
+/**
+ * Typically called by `lookup_pending_webhooks`.
+ *
+ * @param cls a `json_t *` JSON array to build
+ * @param webhook_serial reference to the configured webhook template.
+ * @param next_attempt is the time we should make the next request to the webhook.
+ * @param retries how often have we tried this request to the webhook.
+ * @param url to make request to
+ * @param http_method use for the webhook
+ * @param header of the webhook
+ * @param body of the webhook
+ */
+typedef void
+(*TALER_MERCHANTDB_PendingWebhooksCallback)(void *cls,
+                                            uint64_t webhook_serial,
+                                            struct GNUNET_TIME_Absolute
+                                            next_attempt,
+                                            uint32_t retries,
+                                            const char *url,
+                                            const char *http_method,
+                                            const char *header,
+                                            const char *body);
+
+
+/**
+ * Details about the pending webhook.
+ */
+struct TALER_MERCHANTDB_PendingWebhookDetails
+{
+  /**
+   * Identifies when we should make the next request to the webhook. 0 for unknown,
+   * #GNUNET_TIME_UNIT_FOREVER_ABS for never.
+   */
+  struct GNUNET_TIME_Absolute next_attempt;
+
+  /**
+   * How often have we tried this request so far.
+   */
+  uint32_t retries;
+
+  /**
+   * URL of the webhook. The customer will be redirected on this url.
+   */
+  char *url;
+
+  /**
+   * Http method used for the webhook.
+   */
+  char *http_method;
+
+
+  /**
+   * Header of the webhook.
+   */
+  char *header;
+
+
+  /**
+   * Body of the webhook.
+   */
+  char *body;
+
+};
+
+
 /**
  * Filter preferences.
  */
@@ -2385,6 +2553,297 @@ struct TALER_MERCHANTDB_Plugin
     uint32_t offset,
     const struct TALER_BlindedDenominationSignature *blind_sig);
 
+
+  /**
+   * Lookup all of the templates the given instance has configured.
+   *
+   * @param cls closure
+   * @param instance_id instance to lookup template for
+   * @param cb function to call on all template found
+   * @param cb_cls closure for @a cb
+   * @return database result code
+   */
+  enum GNUNET_DB_QueryStatus
+  (*lookup_templates)(void *cls,
+                      const char *instance_id,
+                      TALER_MERCHANTDB_TemplatesCallback cb,
+                      void *cb_cls);
+
+
+  /**
+   * Lookup details about a particular template.
+   *
+   * @param cls closure
+   * @param instance_id instance to lookup template for
+   * @param template_id template to lookup
+   * @param[out] td set to the template details on success, can be NULL
+   *             (in that case we only want to check if the template exists)
+   * @return database result code
+   */
+  enum GNUNET_DB_QueryStatus
+  (*lookup_template)(void *cls,
+                     const char *instance_id,
+                     const char *template_id,
+                     struct TALER_MERCHANTDB_TemplateDetails *td);
+
+  /**
+   * Delete information about a template.
+   *
+   * @param cls closure
+   * @param instance_id instance to delete template of
+   * @param template_id template to delete
+   * @return DB status code, #GNUNET_DB_STATUS_SUCCESS_NO_RESULTS
+   *           if template unknown.
+   */
+  enum GNUNET_DB_QueryStatus
+  (*delete_template)(void *cls,
+                     const char *instance_id,
+                     const char *template_id);
+
+
+  /**
+   * Insert details about a particular template.
+   *
+   * @param cls closure
+   * @param instance_id instance to insert template for
+   * @param template_id template identifier of template to insert
+   * @param td the template details to insert
+   * @return database result code
+   */
+  enum GNUNET_DB_QueryStatus
+  (*insert_template)(void *cls,
+                     const char *instance_id,
+                     const char *template_id,
+                     const struct TALER_MERCHANTDB_TemplateDetails *td);
+
+
+  /**
+   * Update details about a particular template.
+   *
+   * @param cls closure
+   * @param instance_id instance to update template for
+   * @param template_id template to update
+   * @param td update to the template details on success, can be NULL
+   *             (in that case we only want to check if the template exists)
+   * @return database result code, #GNUNET_DB_STATUS_SUCCESS_NO_RESULTS if the template
+   *         does not yet exist.
+   */
+  enum GNUNET_DB_QueryStatus
+  (*update_template)(void *cls,
+                     const char *instance_id,
+                     const char *template_id,
+                     const struct TALER_MERCHANTDB_TemplateDetails *td);
+
+
+  /**
+   * Lookup all of the webhooks the given instance has configured.
+   *
+   * @param cls closure
+   * @param instance_id instance to lookup webhook for
+   * @param cb function to call on all webhook found
+   * @param cb_cls closure for @a cb
+   * @return database result code
+   */
+  enum GNUNET_DB_QueryStatus
+  (*lookup_webhooks)(void *cls,
+                     const char *instance_id,
+                     TALER_MERCHANTDB_WebhooksCallback cb,
+                     void *cb_cls);
+
+
+  /**
+   * Lookup details about a particular webhook.
+   *
+   * @param cls closure
+   * @param instance_id instance to lookup webhook for
+   * @param webhook_id webhook to lookup
+   * @param[out] wb set to the webhook details on success, can be NULL
+   *             (in that case we only want to check if the webhook exists)
+   * @return database result code
+   */
+  enum GNUNET_DB_QueryStatus
+  (*lookup_webhook)(void *cls,
+                    const char *instance_id,
+                    const char *webhook_id,
+                    struct TALER_MERCHANTDB_WebhookDetails *wb);
+
+  /**
+   * Delete information about a webhook.
+   *
+   * @param cls closure
+   * @param instance_id instance to delete webhook of
+   * @param webhook_id webhook to delete
+   * @return DB status code, #GNUNET_DB_STATUS_SUCCESS_NO_RESULTS
+   *           if webhook unknown.
+   */
+  enum GNUNET_DB_QueryStatus
+  (*delete_webhook)(void *cls,
+                    const char *instance_id,
+                    const char *webhook_id);
+
+
+  /**
+   * Insert details about a particular webhook.
+   *
+   * @param cls closure
+   * @param instance_id instance to insert webhook for
+   * @param webhook_id webhook identifier of webhook to insert
+   * @param wb the webhook details to insert
+   * @return database result code
+   */
+  enum GNUNET_DB_QueryStatus
+  (*insert_webhook)(void *cls,
+                    const char *instance_id,
+                    const char *webhook_id,
+                    const struct TALER_MERCHANTDB_WebhookDetails *wb);
+
+
+  /**
+   * Update details about a particular webhook.
+   *
+   * @param cls closure
+   * @param instance_id instance to update webhook for
+   * @param webhook_id webhook to update
+   * @param wb update to the webhook details on success, can be NULL
+   *             (in that case we only want to check if the webhook exists)
+   * @return database result code, #GNUNET_DB_STATUS_SUCCESS_NO_RESULTS if the webhook
+   *         does not yet exist.
+   */
+  enum GNUNET_DB_QueryStatus
+  (*update_webhook)(void *cls,
+                    const char *instance_id,
+                    const char *webhook_id,
+                    const struct TALER_MERCHANTDB_WebhookDetails *wb);
+
+  /**
+   * Lookup webhook by event
+   *
+   * @param cls closure
+   * @param instance_id instance to lookup webhook for
+   * @param event_type event that we need to put in the pending webhook
+   * @param[out] cb set to the webhook details on success
+   * @param cb_cls callback closure
+   * @return database result code
+   */
+  enum GNUNET_DB_QueryStatus
+  (*lookup_webhook_by_event)(void *cls,
+                             const char *instance_id,
+                             const char *event_type,
+                             TALER_MERCHANTDB_WebhookDetailCallback cb,
+                             void *cb_cls);
+
+  /**
+   * Insert webhook in the pending webhook.
+   *
+   * @param cls closure
+   * @param instance_id instance to insert webhook for
+   * @param webhook_serial webhook to insert in the pending webhook
+   * @param url to make the request to
+   * @param http_method for the webhook
+   * @param header of the webhook
+   * @param body of the webhook
+   * @return database result code
+   */
+  enum GNUNET_DB_QueryStatus
+  (*insert_pending_webhook)(void *cls,
+                            const char *instance_id,
+                            uint64_t webhook_serial,
+                            const char *url,
+                            const char *http_method,
+                            const char *header,
+                            const char *body);
+
+  /**
+   * Lookup details about a particular pending webhook.
+   *
+   * @param cls closure
+   * @param instance_id instance to lookup webhook for
+   * @param webhook_serial webhook to lookup
+   * @param[out] pwb set to the pending webhook details on success, can be NULL
+   *             (in that case we only want to check if the webhook exists)
+   * @return database result code
+   */
+  enum GNUNET_DB_QueryStatus
+  (*lookup_pending_webhook)(void *cls,
+                            const char *instance_id,
+                            uint64_t *webhook_serial,
+                            struct TALER_MERCHANTDB_PendingWebhookDetails *pwb);
+  /**
+   * Lookup the webhook that need to be send in priority. These webhooks are not successfully
+   * send.
+   *
+   * @param cls closure
+   * @param cb pending webhook callback
+   * @param cb_cls callback closure
+   */
+  // WHERE next_attempt <= now ORDER BY next_attempt ASC
+  enum GNUNET_DB_QueryStatus
+  (*lookup_pending_webhooks)(void *cls,
+                             TALER_MERCHANTDB_PendingWebhooksCallback cb,
+                             void *cb_cls);
+
+  /**
+   * Lookup future webhook in the pending webhook that need to be send.
+   * With that we can know how long the system can 'sleep'.
+   *
+   * @param cls closure
+   * @param cb pending webhook callback
+   * @param cb_cls callback closure
+   */
+  // ORDER BY next_attempt ASC LIMIT 1
+  enum GNUNET_DB_QueryStatus
+  (*lookup_future_webhook)(void *cls,
+                           TALER_MERCHANTDB_PendingWebhooksCallback cb,
+                           void *cb_cls);
+
+  /**
+   * Lookup all the webhooks in the pending webhook.
+   * Use by the administrator
+   *
+   * @param cls closure
+   * @param instance_id to lookup webhooks for this instance particularly
+   * @param min_row to see the list of the pending webhook that it is started with this minimum row.
+   * @param max_results to see the list of the pending webhook that it is end with this max results.
+   * @param cb pending webhook callback
+   * @param cb_cls callback closure
+   */
+  // WHERE webhook_pending_serial > min_row ORDER BY webhook_pending_serial ASC LIMIT max_results
+  enum GNUNET_DB_QueryStatus
+  (*lookup_all_webhooks)(void *cls,
+                         const char *instance_id,
+                         uint64_t min_row,
+                         uint32_t max_results,
+                         TALER_MERCHANTDB_PendingWebhooksCallback cb,
+                         void *cb_cls);
+
+
+  /**
+   * Update the pending webhook. It is use if the webhook can't be send.
+   *
+   * @param cls closure
+   * @param webhook_serial webhook that need to be update
+   * @param next_attempt when we should make the next request to the webhook
+   * @return database result code
+   */
+  enum GNUNET_DB_QueryStatus
+  (*update_pending_webhook)(void *cls,
+                            uint64_t webhook_serial,
+                            struct GNUNET_TIME_Absolute next_attempt);
+                            // maybe add: http status of failure?
+
+
+  /**
+   * Delete a webhook in the pending webhook after the
+   * webhook was completed successfully.
+   *
+   * @param cls closure
+   * @param webhook_serial webhook that need to be delete in the pending webhook
+   * @return database result code
+   */
+  enum GNUNET_DB_QueryStatus
+  (*delete_pending_webhook)(void *cls,
+                            uint64_t webhook_serial);
+
 };
 
 #endif
