@@ -26,6 +26,7 @@
 #include <gnunet/gnunet_util_lib.h>
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_merchant_service.h"
+#include "merchant_api_curl_defaults.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
 
@@ -89,7 +90,7 @@ parse_tips (const json_t *ia,
       GNUNET_JSON_spec_fixed_auto ("tip_id",
                                    &ie->tip_id),
       TALER_JSON_spec_amount_any ("tip_amount",
-                              &ie->tip_amount),
+                                  &ie->tip_amount),
       GNUNET_JSON_spec_end ()
     };
 
@@ -286,11 +287,7 @@ TALER_MERCHANT_tips_get2 (struct GNUNET_CURL_Context *ctx,
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Requesting URL '%s'\n",
               tgh->url);
-  eh = curl_easy_init ();
-  GNUNET_assert (CURLE_OK ==
-                 curl_easy_setopt (eh,
-                                   CURLOPT_URL,
-                                   tgh->url));
+  eh = TALER_MERCHANT_curl_easy_get_ (tgh->url);
   tgh->job = GNUNET_CURL_job_add (ctx,
                                   eh,
                                   &handle_get_tips_finished,
