@@ -1,6 +1,6 @@
 /*
   This file is part of TALER
-  Copyright (C) 2014-2021 Taler Systems SA
+  Copyright (C) 2014-2023 Taler Systems SA
 
   TALER is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as
@@ -30,6 +30,7 @@
 #include <gnunet/gnunet_curl_lib.h>
 #include "taler_merchant_service.h"
 #include "merchant_api_curl_defaults.h"
+#include "merchant_api_common.h"
 #include <taler/taler_json_lib.h>
 #include <taler/taler_signatures.h>
 #include <taler/taler_curl_lib.h>
@@ -88,132 +89,12 @@ handle_post_order_finished (void *cls,
 {
   struct TALER_MERCHANT_PostOrdersHandle *po = cls;
   const json_t *json = response;
-  struct TALER_MERCHANT_PostOrdersReply por = {
-    .hr.http_status = (unsigned int) response_code,
-    .hr.reply = json
-  };
-  struct TALER_ClaimTokenP token;
 
   po->job = NULL;
-  switch (response_code)
-  {
-  case 0:
-    por.hr.ec = TALER_EC_GENERIC_INVALID_RESPONSE;
-    break;
-  case MHD_HTTP_OK:
-    {
-      bool no_token;
-      struct GNUNET_JSON_Specification spec[] = {
-        GNUNET_JSON_spec_string ("order_id",
-                                 &por.details.ok.order_id),
-        GNUNET_JSON_spec_mark_optional (
-          GNUNET_JSON_spec_fixed_auto ("token",
-                                       &token),
-          &no_token),
-        GNUNET_JSON_spec_end ()
-      };
-
-      if (GNUNET_OK !=
-          GNUNET_JSON_parse (json,
-                             spec,
-                             NULL, NULL))
-      {
-        GNUNET_break_op (0);
-        por.hr.http_status = 0;
-        por.hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
-      }
-      else
-      {
-        if (! no_token)
-          por.details.ok.token = &token;
-      }
-    }
-    break;
-  case MHD_HTTP_BAD_REQUEST:
-    json_dumpf (json,
-                stderr,
-                JSON_INDENT (2));
-    por.hr.ec = TALER_JSON_get_error_code (json);
-    por.hr.hint = TALER_JSON_get_error_hint (json);
-    /* This should never happen, either us or
-       the merchant is buggy (or API version conflict);
-       just pass JSON reply to the application */
-    break;
-  case MHD_HTTP_UNAUTHORIZED:
-    por.hr.ec = TALER_JSON_get_error_code (json);
-    por.hr.hint = TALER_JSON_get_error_hint (json);
-    /* Nothing really to verify, merchant says we need to authenticate. */
-    break;
-  case MHD_HTTP_FORBIDDEN:
-    /* Nothing really to verify, merchant says one
-       of the signatures is invalid; as we checked them,
-       this should never happen, we should pass the JSON
-       reply to the application */
-    por.hr.ec = TALER_JSON_get_error_code (json);
-    por.hr.hint = TALER_JSON_get_error_hint (json);
-    break;
-  case MHD_HTTP_NOT_FOUND:
-    /* Nothing really to verify, this should never
-       happen, we should pass the JSON reply to the application */
-    por.hr.ec = TALER_JSON_get_error_code (json);
-    por.hr.hint = TALER_JSON_get_error_hint (json);
-    break;
-  case MHD_HTTP_CONFLICT:
-    por.hr.ec = TALER_JSON_get_error_code (json);
-    por.hr.hint = TALER_JSON_get_error_hint (json);
-    break;
-  case MHD_HTTP_GONE:
-    /* The quantity of some product requested was not available. */
-    {
-
-      struct GNUNET_JSON_Specification spec[] = {
-        GNUNET_JSON_spec_string (
-          "product_id",
-          &por.details.gone.product_id),
-        GNUNET_JSON_spec_uint64 (
-          "requested_quantity",
-          &por.details.gone.requested_quantity),
-        GNUNET_JSON_spec_uint64 (
-          "available_quantity",
-          &por.details.gone.available_quantity),
-        GNUNET_JSON_spec_mark_optional (
-          GNUNET_JSON_spec_timestamp (
-            "restock_expected",
-            &por.details.gone.restock_expected),
-          NULL),
-        GNUNET_JSON_spec_end ()
-      };
-
-      if (GNUNET_OK !=
-          GNUNET_JSON_parse (json,
-                             spec,
-                             NULL, NULL))
-      {
-        GNUNET_break_op (0);
-        por.hr.http_status = 0;
-        por.hr.ec = TALER_EC_GENERIC_REPLY_MALFORMED;
-      }
-      break;
-    }
-  case MHD_HTTP_INTERNAL_SERVER_ERROR:
-    /* Server had an internal issue; we should retry,
-       but this API leaves this to the application */
-    por.hr.ec = TALER_JSON_get_error_code (json);
-    por.hr.hint = TALER_JSON_get_error_hint (json);
-    break;
-  default:
-    /* unexpected response code */
-    por.hr.ec = TALER_JSON_get_error_code (json);
-    por.hr.hint = TALER_JSON_get_error_hint (json);
-    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
-                "Unexpected response code %u/%d\n",
-                (unsigned int) response_code,
-                (int) por.hr.ec);
-    GNUNET_break_op (0);
-    break;
-  }
-  po->cb (po->cb_cls,
-          &por);
+  TALER_MERCHANT_handle_order_creation_response_ (po->cb,
+                                                            po->cb_cls,
+                                                            response_code,
+                                                            json);
   TALER_MERCHANT_orders_post_cancel (po);
 }
 
