@@ -51,8 +51,8 @@ function cleanup()
 }
 
 NEXUS_PORT=8082
-SANDBOX_URL=http://localhost:1$NEXUS_PORT/demobanks/default
-export BANK_URL=$SANDBOX_URL
+SANDBOX_URL=http://localhost:1$NEXUS_PORT
+export BANK_URL=$SANDBOX_URL/demobanks/default
 function get_payto_uri() {
     export LIBEUFIN_SANDBOX_USERNAME=$1
     export LIBEUFIN_SANDBOX_PASSWORD=$2
@@ -148,11 +148,11 @@ taler-exchange-httpd --version
 taler-merchant-httpd --version
 libeufin-cli --version
 
-# euFin before the exchange, because must set
+# Setup libeuFin before the exchange, because we must set
 # the PAYTO_URI config option (with unguessable IBAN)
 # before the exchange starts.
-echo "Setting up euFin ..."
-# reset eufin database
+echo "Setting up libeufin ..."
+# reset libeufin database
 rm -f $TALER_DB
 taler-bank-manage-testing $NEXUS_PORT $TALER_DB $EXCHANGE_URL $CONF
 echo -n "Setting up exchange ..."
