@@ -1293,6 +1293,214 @@ run (void *cls,
     TALER_TESTING_cmd_end ()
   };
 
+  struct TALER_TESTING_Command templates[] = {
+    TALER_TESTING_cmd_merchant_get_templates ("get-templates-empty",
+                                              merchant_url,
+                                              MHD_HTTP_OK,
+                                              NULL),
+    TALER_TESTING_cmd_merchant_post_templates ("post-templates-t1",
+                                               merchant_url,
+                                               "template-1",
+                                               "a template",
+                                               MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_post_templates ("post-templates-t1-idem",
+                                               merchant_url,
+                                               "template-1",
+                                               "a template",
+                                               MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_post_templates ("post-templates-t1-non-idem",
+                                               merchant_url,
+                                               "template-1",
+                                               "a different template",
+                                               MHD_HTTP_CONFLICT),
+    TALER_TESTING_cmd_merchant_get_templates ("get-templates-t1",
+                                              merchant_url,
+                                              MHD_HTTP_OK,
+                                              "post-templates-t1",
+                                              NULL),
+    TALER_TESTING_cmd_merchant_get_template ("get-template-t1",
+                                             merchant_url,
+                                             "template-1",
+                                             MHD_HTTP_OK,
+                                             "post-templates-t1"),
+    TALER_TESTING_cmd_merchant_post_templates ("post-templates-t2",
+                                               merchant_url,
+                                               "template-2",
+                                               "a template",
+                                               MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_patch_template (
+      "patch-templates-t2",
+      merchant_url,
+      "template-2",
+      "another template",
+      "data:image/jpeg;base64,RAWDATA",
+      GNUNET_JSON_PACK (
+      GNUNET_JSON_pack_uint64 ("minimum_age", 0),
+      GNUNET_JSON_pack_time_rel ("pay_duration",
+                                 GNUNET_TIME_UNIT_MINUTES)),
+      MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_get_template ("get-template-t2",
+                                             merchant_url,
+                                             "template-2",
+                                             MHD_HTTP_OK,
+                                             "patch-templates-t2"),
+    TALER_TESTING_cmd_merchant_get_template ("get-template-nx",
+                                             merchant_url,
+                                             "template-nx",
+                                             MHD_HTTP_NOT_FOUND,
+                                             NULL),
+    TALER_TESTING_cmd_merchant_patch_template (
+      "patch-templates-t3-nx",
+      merchant_url,
+      "template-3",
+      "updated template",
+      "data:image/jpeg;base64,RAWDATA",
+      GNUNET_JSON_PACK (
+      GNUNET_JSON_pack_uint64 ("minimum_age", 0),
+      GNUNET_JSON_pack_time_rel ("pay_duration",
+                                 GNUNET_TIME_UNIT_MINUTES)),
+      MHD_HTTP_NOT_FOUND),
+    TALER_TESTING_cmd_merchant_post_templates2 (
+      "post-templates-t3-amount",
+      merchant_url,
+      "template-amount",
+      "a different template with an amount",
+      NULL,
+      GNUNET_JSON_PACK (
+                        GNUNET_JSON_pack_uint64 ("minimum_age", 0),
+                        GNUNET_JSON_pack_time_rel ("pay_duration",
+                                                   GNUNET_TIME_UNIT_MINUTES),
+                        GNUNET_JSON_pack_string ("amount",
+                                                 "EUR:4")),
+      MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_post_using_templates (
+      "using-templates-t1",
+      "post-templates-t1",
+      merchant_url,
+      "summary-1",
+      "EUR:10",
+      MHD_HTTP_OK),
+    TALER_TESTING_cmd_merchant_post_using_templates (
+      "using-templates-t1-amount-missing",
+      "post-templates-t1",
+      merchant_url,
+      "summary-1",
+      NULL,
+      MHD_HTTP_CONFLICT),
+    TALER_TESTING_cmd_merchant_post_using_templates (
+      "using-templates-t1-summary-missing",
+      "post-templates-t1",
+      merchant_url,
+      NULL,
+      "EUR:10",
+      MHD_HTTP_CONFLICT),
+    TALER_TESTING_cmd_merchant_post_using_templates (
+      "using-templates-t1-amount-conflict",
+      "post-templates-t3-amount",
+      merchant_url,
+      "summary-1",
+      "EUR:10",
+      MHD_HTTP_CONFLICT),
+    TALER_TESTING_cmd_merchant_post_using_templates (
+      "using-templates-t1-amount-duplicate",
+      "post-templates-t3-amount",
+      merchant_url,
+      "summary-1",
+      "EUR:4",
+      MHD_HTTP_CONFLICT),
+    TALER_TESTING_cmd_merchant_delete_template ("get-templates-empty",
+                                                merchant_url,
+                                                "t1",
+                                                MHD_HTTP_NOT_FOUND),
+    TALER_TESTING_cmd_merchant_delete_template ("get-templates-empty",
+                                                merchant_url,
+                                                "template-1",
+                                                MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_post_using_templates (
+      "post-templates-t1-deleted",
+      "post-templates-t1",
+      merchant_url,
+      "summary-1",
+      "EUR:5",
+      MHD_HTTP_NOT_FOUND),
+    TALER_TESTING_cmd_end ()
+  };
+
+  struct TALER_TESTING_Command webhooks[] = {
+    TALER_TESTING_cmd_merchant_get_webhooks ("get-webhooks-empty",
+                                             merchant_url,
+                                             MHD_HTTP_OK,
+                                             NULL),
+    TALER_TESTING_cmd_merchant_post_webhooks ("post-webhooks-w1",
+                                              merchant_url,
+                                              "webhook-1",
+                                              "Paid",
+                                              MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_post_webhooks ("post-webhooks-w1-idem",
+                                              merchant_url,
+                                              "webhook-1",
+                                              "Paid",
+                                              MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_post_webhooks ("post-webhooks-w1-non-idem",
+                                              merchant_url,
+                                              "webhook-1",
+                                              "Refund",
+                                              MHD_HTTP_CONFLICT),
+    TALER_TESTING_cmd_merchant_get_webhooks ("get-webhooks-w1",
+                                             merchant_url,
+                                             MHD_HTTP_OK,
+                                             "post-webhooks-w1",
+                                             NULL),
+    TALER_TESTING_cmd_merchant_get_webhook ("get-webhook-w1",
+                                            merchant_url,
+                                            "webhook-1",
+                                            MHD_HTTP_OK,
+                                            "post-webhooks-w1"),
+    TALER_TESTING_cmd_merchant_post_webhooks ("post-webhooks-w2",
+                                              merchant_url,
+                                              "webhook-2",
+                                              "Paid",
+                                              MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_patch_webhook ("patch-webhooks-w2",
+                                              merchant_url,
+                                              "webhook-2",
+                                              "Refund2",
+                                              "https://example.com",
+                                              "POST",
+                                              "Authorization:WHWOXZXPLL",
+                                              "Amount",
+                                              MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_merchant_get_webhook ("get-webhook-w2",
+                                            merchant_url,
+                                            "webhook-2",
+                                            MHD_HTTP_OK,
+                                            "patch-webhooks-w2"),
+    TALER_TESTING_cmd_merchant_get_webhook ("get-webhook-nx",
+                                            merchant_url,
+                                            "webhook-nx",
+                                            MHD_HTTP_NOT_FOUND,
+                                            NULL),
+    TALER_TESTING_cmd_merchant_patch_webhook ("patch-webhooks-w3-nx",
+                                              merchant_url,
+                                              "webhook-3",
+                                              "Paid2",
+                                              "https://example.com",
+                                              "POST",
+                                              "Authorization:WHWOXZXPLL",
+                                              "Amount",
+                                              MHD_HTTP_NOT_FOUND),
+    TALER_TESTING_cmd_merchant_delete_webhook ("get-webhooks-empty",
+                                               merchant_url,
+                                               "w1",
+                                               MHD_HTTP_NOT_FOUND),
+    TALER_TESTING_cmd_merchant_delete_webhook ("get-webhooks-empty",
+                                               merchant_url,
+                                               "webhook-1",
+                                               MHD_HTTP_NO_CONTENT),
+    TALER_TESTING_cmd_end ()
+  };
+
+
   struct TALER_TESTING_Command commands[] = {
     /* general setup */
     TALER_TESTING_cmd_auditor_add ("add-auditor-OK",
@@ -1643,6 +1851,10 @@ run (void *cls,
                              tip),
     TALER_TESTING_cmd_batch ("auth",
                              auth),
+    TALER_TESTING_cmd_batch ("templates",
+                             templates),
+    TALER_TESTING_cmd_batch ("webhooks",
+                             webhooks),
     /**
      * End the suite.
      */
