@@ -19,8 +19,8 @@
 
 . initialize_taler_system.sh
 
-echo -n "Configuring merchant instance ..."
 FORTYTHREE=`get_payto_uri fortythree x`
+echo -n "Configuring merchant instance ..."
 STATUS=$(curl -H "Content-Type: application/json" -X POST \
     -H 'Authorization: Bearer secret-token:super_secret' \
     http://localhost:9966/management/instances \
@@ -29,7 +29,7 @@ STATUS=$(curl -H "Content-Type: application/json" -X POST \
 
 if [ "$STATUS" != "204" ]
 then
-    echo 'should respond ok, instance created. got:' $STATUS
+    echo "Should have responded 204 No content, instance created. Got $STATUS instead"
     exit 1
 fi
 echo OK
@@ -164,16 +164,16 @@ STATUS=$(curl 'http://localhost:9966/instances/default/private/orders' \
 
 if [ "$STATUS" != "200" ]
 then
-    echo 'should respond ok, lock should apply. got:' $STATUS `cat $LAST_RESPONSE`
+    echo 'Should respond 200 OK, lock should apply. Got:' $STATUS `cat $LAST_RESPONSE`
     exit 1
 fi
 
 
 echo " OK"
 
-echo -n "First withdraw wallet"
+echo -n "First withdraw wallet ..."
 rm $WALLET_DB
-taler-wallet-cli --no-throttle --wallet-db=$WALLET_DB api 'withdrawTestBalance' \
+taler-wallet-cli --no-throttle --wallet-db=$WALLET_DB api --expect-success 'withdrawTestBalance' \
   "$(jq -n '
     {
         amount: "TESTKUDOS:5",
@@ -183,12 +183,12 @@ taler-wallet-cli --no-throttle --wallet-db=$WALLET_DB api 'withdrawTestBalance'
     --arg BANK_URL "$BANK_URL/access-api/" \
     --arg EXCHANGE_URL "$EXCHANGE_URL"
   )" 2>wallet-withdraw-1.err >wallet-withdraw-1.out
+echo -n "."
 taler-wallet-cli --wallet-db=$WALLET_DB run-until-done 2>wallet-withdraw-finish-1.err >wallet-withdraw-finish-1.out
 echo " OK"
 
-NOW=`date +%s`
-
 echo -n "Pay first order ..."
+NOW=`date +%s`
 taler-wallet-cli --no-throttle --wallet-db=$WALLET_DB handle-uri "${PAY_URL}" -y 2> wallet-pay1.err > wallet-pay1.log
 NOW2=`date +%s`
 echo " OK (took $( echo -n $(($NOW2 - $NOW)) ) secs )"
