@@ -113,11 +113,11 @@ get_instances_cb (void *cls,
         gis->instances[i]);
 
       {
-        const char *name;
+        const char **name;
+
         if (GNUNET_OK !=
-            TALER_TESTING_get_trait_string (instance_cmd,
-                                            0,
-                                            &name))
+            TALER_TESTING_get_trait_instance_name (instance_cmd,
+                                                   &name))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Could not fetch instance name\n");
@@ -125,7 +125,7 @@ get_instances_cb (void *cls,
           return;
         }
         if (0 != strcmp (iis[i].name,
-                         name))
+                         *name))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Instance name does not match\n");
@@ -135,11 +135,11 @@ get_instances_cb (void *cls,
       }
 
       {
-        const char *id;
+        const char **id;
+
         if (GNUNET_OK !=
-            TALER_TESTING_get_trait_string (instance_cmd,
-                                            1,
-                                            &id))
+            TALER_TESTING_get_trait_instance_id (instance_cmd,
+                                                 &id))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Could not fetch instance id\n");
@@ -147,7 +147,7 @@ get_instances_cb (void *cls,
           return;
         }
         if (0 != strcmp (iis[i].id,
-                         id))
+                         *id))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Instance id does not match\n");
