@@ -83,7 +83,6 @@ get_reserve_cb (void *cls,
                 unsigned int tips_length,
                 const struct TALER_MERCHANT_TipDetails tips[])
 {
-  /* FIXME, deeper checks should be implemented here. */
   struct GetReserveState *grs = cls;
   const struct TALER_TESTING_Command *reserve_cmd;
 
@@ -105,17 +104,15 @@ get_reserve_cb (void *cls,
   switch (hr->http_status)
   {
   case MHD_HTTP_OK:
-    // FIXME: use grs->reserve_reference here to
-    // check if the data returned matches that from the POST / PATCH
     {
       const struct TALER_Amount *initial_amount;
       if (GNUNET_OK !=
-          TALER_TESTING_get_trait_amount_obj (reserve_cmd,
-                                              0,
-                                              &initial_amount))
+          TALER_TESTING_get_trait_amount (reserve_cmd,
+                                          &initial_amount))
         TALER_TESTING_interpreter_fail (grs->is);
-      if ((GNUNET_OK != TALER_amount_cmp_currency (&rs->merchant_initial_amount,
-                                                   initial_amount)) ||
+      if ((GNUNET_OK !=
+           TALER_amount_cmp_currency (&rs->merchant_initial_amount,
+                                      initial_amount)) ||
           (0 != TALER_amount_cmp (&rs->merchant_initial_amount,
                                   initial_amount)))
       {
@@ -139,11 +136,10 @@ get_reserve_cb (void *cls,
       tip_cmd = TALER_TESTING_interpreter_lookup_command (grs->is,
                                                           grs->tips[i]);
       {
-        const struct GNUNET_HashCode *tip_id;
+        const struct TALER_TipIdentifierP *tip_id;
 
         if (GNUNET_OK !=
             TALER_TESTING_get_trait_tip_id (tip_cmd,
-                                            0,
                                             &tip_id))
           TALER_TESTING_interpreter_fail (grs->is);
 
@@ -160,13 +156,13 @@ get_reserve_cb (void *cls,
         const struct TALER_Amount *total_amount;
 
         if (GNUNET_OK !=
-            TALER_TESTING_get_trait_amount_obj (tip_cmd,
-                                                0,
-                                                &total_amount))
+            TALER_TESTING_get_trait_amount (tip_cmd,
+                                            &total_amount))
           TALER_TESTING_interpreter_fail (grs->is);
 
-        if ((GNUNET_OK != TALER_amount_cmp_currency (&tips[i].amount,
-                                                     total_amount)) ||
+        if ((GNUNET_OK !=
+             TALER_amount_cmp_currency (&tips[i].amount,
+                                        total_amount)) ||
             (0 != TALER_amount_cmp (&tips[i].amount,
                                     total_amount)))
         {
@@ -177,16 +173,15 @@ get_reserve_cb (void *cls,
         }
       }
       {
-        const char *reason;
+        const char **reason;
 
         if (GNUNET_OK !=
-            TALER_TESTING_get_trait_string (tip_cmd,
-                                            0,
+            TALER_TESTING_get_trait_reason (tip_cmd,
                                             &reason))
           TALER_TESTING_interpreter_fail (grs->is);
 
         if (0 != strcmp (tips[i].reason,
-                         reason))
+                         *reason))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Reserve tip reason does not match\n");
@@ -225,7 +220,6 @@ get_reserve_run (void *cls,
     grs->reserve_reference);
   if (GNUNET_OK !=
       TALER_TESTING_get_trait_reserve_pub (reserve_cmd,
-                                           0,
                                            &reserve_pub))
     TALER_TESTING_FAIL (is);
 
