@@ -215,7 +215,7 @@ post_instances_run (void *cls,
  * @param index index number of the object to extract.
  * @return #GNUNET_OK on success
  */
-static int
+static enum GNUNET_GenericReturnValue
 post_instances_traits (void *cls,
                        const void **ret,
                        const char *trait,
@@ -225,32 +225,33 @@ post_instances_traits (void *cls,
   #define NUM_TRAITS (pis->payto_uris_length) + 11
   struct TALER_TESTING_Trait traits[NUM_TRAITS];
   traits[0] =
-    TALER_TESTING_make_trait_string (0, pis->name);
+    TALER_TESTING_make_trait_instance_name (&pis->name);
   traits[1] =
-    TALER_TESTING_make_trait_string (1, pis->instance_id);
+    TALER_TESTING_make_trait_instance_id (&pis->instance_id);
   traits[2] =
-    TALER_TESTING_make_trait_json (0, pis->address);
+    TALER_TESTING_make_trait_address (pis->address);
   traits[3] =
-    TALER_TESTING_make_trait_json (1, pis->jurisdiction);
+    TALER_TESTING_make_trait_jurisdiction (pis->jurisdiction);
   traits[4] =
-    TALER_TESTING_make_trait_amount_obj (0, &pis->default_max_wire_fee);
+    TALER_TESTING_make_trait_max_wire_fee (&pis->default_max_wire_fee);
   traits[5] =
-    TALER_TESTING_make_trait_uint32 (0, &pis->default_wire_fee_amortization);
+    TALER_TESTING_make_trait_wire_fee_amortization (
+      &pis->default_wire_fee_amortization);
   traits[6] =
-    TALER_TESTING_make_trait_amount_obj (1, &pis->default_max_deposit_fee);
+    TALER_TESTING_make_trait_max_deposit_fee (&pis->default_max_deposit_fee);
   traits[7] =
-    TALER_TESTING_make_trait_relative_time (0,
-                                            &pis->default_wire_transfer_delay);
+    TALER_TESTING_make_trait_wire_delay (&pis->default_wire_transfer_delay);
   traits[8] =
-    TALER_TESTING_make_trait_relative_time (1, &pis->default_pay_delay);
+    TALER_TESTING_make_trait_pay_delay (&pis->default_pay_delay);
   traits[9] =
-    TALER_TESTING_make_trait_uint32 (1, &pis->payto_uris_length);
+    TALER_TESTING_make_trait_payto_length (&pis->payto_uris_length);
   traits[NUM_TRAITS - 1] =
     TALER_TESTING_trait_end ();
   for (unsigned int i = 0; i < pis->payto_uris_length; ++i)
   {
     traits[10 + i] =
-      TALER_TESTING_make_trait_string (2 + i, pis->payto_uris[i]);
+      TALER_TESTING_make_trait_payto_uris (i,
+                                           &pis->payto_uris[i]);
   }
 
   return TALER_TESTING_get_trait (traits,
