@@ -115,35 +115,30 @@ refund_cb (
       for (unsigned int i = 0; i < refunds_length; ++i)
       {
         const struct TALER_TESTING_Command *refund_cmd;
-        const char *expected_amount_str;
-        struct TALER_Amount expected_amount;
+        const struct TALER_Amount *expected_amount;
 
         refund_cmd = TALER_TESTING_interpreter_lookup_command (
           wrs->is,
           wrs->refunds[i]);
 
         if (GNUNET_OK !=
-            TALER_TESTING_get_trait_string (refund_cmd,
-                                            0,
-                                            &expected_amount_str))
+            TALER_TESTING_get_trait_amount (refund_cmd,
+                                            &expected_amount))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                       "Could not fetch refund amount\n");
           TALER_TESTING_interpreter_fail (wrs->is);
           return;
         }
-        GNUNET_assert (GNUNET_OK ==
-                       TALER_string_to_amount (expected_amount_str,
-                                               &expected_amount));
         /* The most recent refunds are returned first */
         GNUNET_assert (0 <= TALER_amount_add (&refunded_total,
                                               &refunded_total,
                                               &refunds[refunds_length - 1
                                                        - i].refund_amount));
         if ((GNUNET_OK !=
-             TALER_amount_cmp_currency (&expected_amount,
+             TALER_amount_cmp_currency (expected_amount,
                                         &refunded_total)) ||
-            (0 != TALER_amount_cmp (&expected_amount,
+            (0 != TALER_amount_cmp (expected_amount,
                                     &refunded_total)))
         {
           GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
@@ -178,14 +173,13 @@ obtain_refunds_run (void *cls,
   const struct TALER_TESTING_Command *proposal_cmd =
     TALER_TESTING_interpreter_lookup_command (is,
                                               wrs->proposal_reference);
-  const struct GNUNET_HashCode *h_contract_terms;
+  const struct TALER_PrivateContractHashP *h_contract_terms;
   const char *order_id;
 
   if (NULL == proposal_cmd)
     TALER_TESTING_FAIL (is);
   if (GNUNET_OK !=
       TALER_TESTING_get_trait_h_contract_terms (proposal_cmd,
-                                                0,
                                                 &h_contract_terms))
     TALER_TESTING_FAIL (is);
 
@@ -196,7 +190,6 @@ obtain_refunds_run (void *cls,
 
     if (GNUNET_OK !=
         TALER_TESTING_get_trait_contract_terms (proposal_cmd,
-                                                0,
                                                 &contract_terms))
       TALER_TESTING_FAIL (is);
     {
